"""
Test user added container commands
"""


import sys
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestCmdContainer(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_container_add(self):
        self.container_add()

    def check_command_tree_exists(self):
        """This makes sure we can still run the command tree we added."""
        self.runCmd("test-multi")
        self.runCmd("test-multi test-multi-sub")
        self.runCmd("test-multi test-multi-sub welcome")

    def container_add(self):
        # Make sure we can't overwrite built-in commands:
        self.expect(
            "command container add process",
            "Can't replace builtin container command",
            substrs=["can't replace builtin command"],
            error=True,
        )
        self.expect(
            "command container add process non_such_subcommand",
            "Can't add to built-in subcommand",
            substrs=["Path component: 'process' is not a user command"],
            error=True,
        )
        self.expect(
            "command container add process launch",
            "Can't replace builtin subcommand",
            substrs=["Path component: 'process' is not a user command"],
            error=True,
        )

        # Now lets make a container command:
        self.runCmd("command container add -h 'A test container command' test-multi")
        # Make sure the help works:
        self.expect(
            "help test-multi",
            "Help works for top-level multi",
            substrs=["A test container command"],
        )
        # Add a subcommand:
        self.runCmd(
            "command container add -h 'A test container sub-command' test-multi test-multi-sub"
        )
        # Make sure the help works:
        self.expect(
            "help test-multi",
            "Help shows sub-multi",
            substrs=[
                "A test container command",
                "test-multi-sub -- A test container sub-command",
            ],
        )
        self.expect(
            "help test-multi test-multi-sub",
            "Help shows sub-multi",
            substrs=["A test container sub-command"],
        )

        # Now add a script based command to the container command:
        self.runCmd("command script import welcome.py")
        self.runCmd(
            "command script add -c welcome.WelcomeCommand test-multi test-multi-sub welcome"
        )
        # Make sure the help still works:
        self.expect(
            "help test-multi test-multi-sub",
            "Listing subcommands works",
            substrs=[
                "A test container sub-command",
                "welcome -- Just a docstring for Welcome",
            ],
        )
        self.expect(
            "help test-multi test-multi-sub welcome",
            "Subcommand help works",
            substrs=["Just a docstring for Welcome"],
        )
        # And make sure it actually works:
        self.expect(
            "test-multi test-multi-sub welcome friend",
            "Test command works",
            substrs=["Hello friend, welcome to LLDB"],
        )

        # Make sure we can make an alias to this:
        self.runCmd(
            "command alias my-welcome test-multi test-multi-sub welcome",
            "We can make an alias to multi-word",
        )
        self.expect(
            "my-welcome friend",
            "Test command works",
            substrs=["Hello friend, welcome to LLDB"],
        )
        self.runCmd("command unalias my-welcome")

        # Make sure overwriting works on the leaf command.  First using the
        # explicit option so we should not be able to remove extant commands by default:

        self.expect(
            "command script add -c welcome.WelcomeCommand2 test-multi test-multi-sub welcome",
            "overwrite command w/o -o",
            substrs=["cannot add command: sub-command already exists"],
            error=True,
        )
        # But we can with the -o option:
        self.runCmd(
            "command script add -c welcome.WelcomeCommand2 -o test-multi test-multi-sub welcome"
        )
        # Make sure we really did overwrite:
        self.expect(
            "test-multi test-multi-sub welcome friend",
            "Used the new command class",
            substrs=["Hello friend, welcome again to LLDB"],
        )
        self.expect(
            "apropos welcome",
            "welcome should show up in apropos",
            substrs=["A docstring for the second Welcome"],
        )
        self.expect(
            "help test-multi test-multi-sub welcome",
            "welcome should show up in help",
            substrs=["A docstring for the second Welcome"],
        )
        self.expect("help", "test-multi should show up in help", substrs=["test-multi"])

        # Now switch the default and make sure we can now delete w/o the overwrite option:
        self.runCmd("settings set interpreter.require-overwrite 0")
        self.runCmd(
            "command script add -c welcome.WelcomeCommand test-multi test-multi-sub welcome"
        )
        # Make sure we really did overwrite:
        self.expect(
            "test-multi test-multi-sub welcome friend",
            "Used the new command class",
            substrs=["Hello friend, welcome to LLDB"],
        )

        # Make sure we give good errors when the input is wrong:
        self.expect(
            "command script delete test-mult test-multi-sub welcome",
            "Delete script command - wrong first path component",
            substrs=["'test-mult' not found"],
            error=True,
        )

        self.expect(
            "command script delete test-multi test-multi-su welcome",
            "Delete script command - wrong second path component",
            substrs=["'test-multi-su' not found"],
            error=True,
        )
        self.check_command_tree_exists()

        self.expect(
            "command script delete test-multi test-multi-sub welcom",
            "Delete script command - wrong leaf component",
            substrs=["'welcom' not found"],
            error=True,
        )
        self.check_command_tree_exists()

        self.expect(
            "command script delete test-multi test-multi-sub",
            "Delete script command - no leaf component",
            substrs=["subcommand 'test-multi-sub' is not a user command"],
            error=True,
        )
        self.check_command_tree_exists()

        # You can't use command script delete to delete container commands:
        self.expect(
            "command script delete test-multi",
            "Delete script - can't delete container",
            substrs=["command 'test-multi' is a multi-word command."],
            error=True,
        )
        self.expect(
            "command script delete test-multi test-multi-sub",
            "Delete script - can't delete container",
            substrs=["subcommand 'test-multi-sub' is not a user command"],
            error=True,
        )

        # You can't use command container delete to delete scripted commands:
        self.expect(
            "command container delete test-multi test-multi-sub welcome",
            "command container can't delete user commands",
            substrs=["subcommand 'welcome' is not a container command"],
            error=True,
        )

        # Also make sure you can't alias on top of container commands:
        self.expect(
            "command alias test-multi process launch",
            "Tried to alias on top of a container command",
            substrs=[
                "'test-multi' is a user container command and cannot be overwritten."
            ],
            error=True,
        )
        self.check_command_tree_exists()

        # Also assert that we can't delete builtin commands:
        self.expect(
            "command script delete process launch",
            "Delete builtin command fails",
            substrs=["command 'process' is not a user command"],
            error=True,
        )
        # Now let's do the version that works
        self.expect(
            "command script delete test-multi test-multi-sub welcome",
            "Delete script command by path",
            substrs=["Deleted command: test-multi test-multi-sub welcome"],
        )

        # Now overwrite the sub-command, it should end up empty:
        self.runCmd(
            "command container add -h 'A different help string' -o test-multi test-multi-sub"
        )
        # welcome should be gone:
        self.expect(
            "test-multi test-multi-sub welcome friend",
            "did remove subcommand",
            substrs=["'test-multi-sub' does not have any subcommands."],
            error=True,
        )
        # We should have the new help:
        self.expect(
            "help test-multi test-multi-sub",
            "help changed",
            substrs=["A different help string"],
        )

        # Now try deleting commands.
        self.runCmd("command container delete test-multi test-multi-sub")
        self.expect(
            "test-multi test-multi-sub",
            "Command is not active",
            error=True,
            substrs=["'test-multi' does not have any subcommands"],
        )
        self.expect("help test-multi", matching=False, substrs=["test-multi-sub"])

        # Next the root command:
        self.runCmd("command container delete test-multi")
        self.expect(
            "test-multi",
            "Root command gone",
            substrs=["'test-multi' is not a valid command."],
            error=True,
        )



"""
This tests that we do not lose control of the inferior, while doing an instruction-level step
over a thread creation instruction.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CreateDuringInstructionStepTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessPlatform(["linux"])
    @expectedFailureAndroid("llvm.org/pr24737", archs=["arm$"])
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"], bugnumber="llvm.org/pr24737")
    def test_step_inst(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target and target.IsValid(), "Target is valid")

        # This should create a breakpoint in the stepping thread.
        breakpoint = target.BreakpointCreateByName("main")
        self.assertTrue(breakpoint and breakpoint.IsValid(), "Breakpoint is valid")

        # Run the program.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process and process.IsValid(), PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.assertEqual(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint)
        self.assertEqual(len(threads), 1, STOPPED_DUE_TO_BREAKPOINT)

        thread = threads[0]
        self.assertTrue(thread and thread.IsValid(), "Thread is valid")

        # Make sure we see only one threads
        self.assertEqual(
            process.GetNumThreads(),
            1,
            "Number of expected threads and actual threads do not match.",
        )

        # Keep stepping until we see the thread creation
        while process.GetNumThreads() < 2:
            thread.StepInstruction(False)
            self.assertEqual(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
            self.assertEqual(
                thread.GetStopReason(),
                lldb.eStopReasonPlanComplete,
                "Step operation succeeded",
            )
            if self.TraceOn():
                self.runCmd("disassemble --pc")

        if self.TraceOn():
            self.runCmd("thread list")

        # We have successfully caught thread creation. Now just run to
        # completion
        process.Continue()

        # At this point, the inferior process should have exited.
        self.assertState(process.GetState(), lldb.eStateExited, PROCESS_EXITED)



"""
Test saving a mini dump.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ProcessSaveCoreMinidumpTestCase(TestBase):
    def verify_core_file(
        self,
        core_path,
        expected_pid,
        expected_modules,
        expected_threads,
        stacks_to_sps_map,
        stacks_to_registers_map,
    ):
        # To verify, we'll launch with the mini dump
        target = self.dbg.CreateTarget(None)
        process = target.LoadCore(core_path)

        # check if the core is in desired state
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertTrue(process.GetProcessInfo().IsValid())
        self.assertEqual(process.GetProcessInfo().GetProcessID(), expected_pid)
        self.assertNotEqual(target.GetTriple().find("linux"), -1)
        self.assertTrue(target.GetNumModules(), len(expected_modules))
        self.assertEqual(process.GetNumThreads(), len(expected_threads))

        for module, expected in zip(target.modules, expected_modules):
            self.assertTrue(module.IsValid())
            module_file_name = module.GetFileSpec().GetFilename()
            expected_file_name = expected.GetFileSpec().GetFilename()
            # skip kernel virtual dynamic shared objects
            if "vdso" in expected_file_name:
                continue
            self.assertEqual(module_file_name, expected_file_name)
            self.assertEqual(module.GetUUIDString(), expected.GetUUIDString())

        red_zone = process.GetTarget().GetStackRedZoneSize()
        for thread_idx in range(process.GetNumThreads()):
            thread = process.GetThreadAtIndex(thread_idx)
            self.assertTrue(thread.IsValid())
            thread_id = thread.GetThreadID()
            self.assertIn(thread_id, expected_threads)
            frame = thread.GetFrameAtIndex(0)
            sp_region = lldb.SBMemoryRegionInfo()
            sp = frame.GetSP()
            err = process.GetMemoryRegionInfo(sp, sp_region)
            self.assertTrue(err.Success(), err.GetCString())
            error = lldb.SBError()
            # Ensure thread_id is in the saved map
            self.assertIn(thread_id, stacks_to_sps_map)
            # Ensure the SP is correct
            self.assertEqual(stacks_to_sps_map[thread_id], sp)
            # Try to read at the end of the stack red zone and succeed
            process.ReadMemory(sp - red_zone, 1, error)
            self.assertTrue(error.Success(), error.GetCString())
            # Try to read just past the red zone and fail
            process.ReadMemory(sp - red_zone - 1, 1, error)
            self.assertTrue(error.Fail(), "No failure when reading past the red zone")
            # Verify the registers are the same
            self.assertIn(thread_id, stacks_to_registers_map)
            register_val_list = stacks_to_registers_map[thread_id]
            frame_register_list = frame.GetRegisters()
            # explicitly verify we collected fs and gs base for x86_64
            explicit_registers = ["fs_base", "gs_base"]
            for reg in explicit_registers:
                register = frame_register_list.GetFirstValueByName(reg)
                self.assertNotEqual(None, register)
                self.assertEqual(
                    register.GetValueAsUnsigned(),
                    stacks_to_registers_map[thread_id]
                    .GetFirstValueByName("fs_base")
                    .GetValueAsUnsigned(),
                )

            for x in register_val_list:
                self.assertEqual(
                    x.GetValueAsUnsigned(),
                    frame_register_list.GetFirstValueByName(
                        x.GetName()
                    ).GetValueAsUnsigned(),
                )

        self.dbg.DeleteTarget(target)

    @skipUnlessArch("x86_64")
    @skipUnlessPlatform(["linux"])
    def test_save_linux_mini_dump(self):
        """Test that we can save a Linux mini dump."""

        self.build()
        exe = self.getBuildArtifact("a.out")
        core_stack = self.getBuildArtifact("core.stack.dmp")
        core_dirty = self.getBuildArtifact("core.dirty.dmp")
        core_full = self.getBuildArtifact("core.full.dmp")
        core_sb_stack = self.getBuildArtifact("core_sb.stack.dmp")
        core_sb_dirty = self.getBuildArtifact("core_sb.dirty.dmp")
        core_sb_full = self.getBuildArtifact("core_sb.full.dmp")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)

            # get neccessary data for the verification phase
            process_info = process.GetProcessInfo()
            expected_pid = process_info.GetProcessID() if process_info.IsValid() else -1
            expected_number_of_modules = target.GetNumModules()
            expected_modules = target.modules
            expected_number_of_threads = process.GetNumThreads()
            expected_threads = []
            stacks_to_sp_map = {}
            stacks_to_registers_map = {}

            for thread_idx in range(process.GetNumThreads()):
                thread = process.GetThreadAtIndex(thread_idx)
                thread_id = thread.GetThreadID()
                expected_threads.append(thread_id)
                stacks_to_sp_map[thread_id] = thread.GetFrameAtIndex(0).GetSP()
                stacks_to_registers_map[thread_id] = thread.GetFrameAtIndex(
                    0
                ).GetRegisters()

            # save core and, kill process and verify corefile existence
            base_command = "process save-core --plugin-name=minidump "
            self.runCmd(base_command + " --style=stack '%s'" % (core_stack))
            self.assertTrue(os.path.isfile(core_stack))
            self.verify_core_file(
                core_stack,
                expected_pid,
                expected_modules,
                expected_threads,
                stacks_to_sp_map,
                stacks_to_registers_map,
            )

            self.runCmd(base_command + " --style=modified-memory '%s'" % (core_dirty))
            self.assertTrue(os.path.isfile(core_dirty))
            self.verify_core_file(
                core_dirty,
                expected_pid,
                expected_modules,
                expected_threads,
                stacks_to_sp_map,
                stacks_to_registers_map,
            )

            self.runCmd(base_command + " --style=full '%s'" % (core_full))
            self.assertTrue(os.path.isfile(core_full))
            self.verify_core_file(
                core_full,
                expected_pid,
                expected_modules,
                expected_threads,
                stacks_to_sp_map,
                stacks_to_registers_map,
            )

            options = lldb.SBSaveCoreOptions()
            core_sb_stack_spec = lldb.SBFileSpec(core_sb_stack)
            options.SetOutputFile(core_sb_stack_spec)
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreStackOnly)
            # validate saving via SBProcess
            error = process.SaveCore(options)
            self.assertTrue(error.Success())
            self.assertTrue(os.path.isfile(core_sb_stack))
            self.verify_core_file(
                core_sb_stack,
                expected_pid,
                expected_modules,
                expected_threads,
                stacks_to_sp_map,
                stacks_to_registers_map,
            )

            options = lldb.SBSaveCoreOptions()
            core_sb_dirty_spec = lldb.SBFileSpec(core_sb_dirty)
            options.SetOutputFile(core_sb_dirty_spec)
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreDirtyOnly)
            error = process.SaveCore(options)
            self.assertTrue(error.Success())
            self.assertTrue(os.path.isfile(core_sb_dirty))
            self.verify_core_file(
                core_sb_dirty,
                expected_pid,
                expected_modules,
                expected_threads,
                stacks_to_sp_map,
                stacks_to_registers_map,
            )

            # Minidump can now save full core files, but they will be huge and
            # they might cause this test to timeout.
            options = lldb.SBSaveCoreOptions()
            core_sb_full_spec = lldb.SBFileSpec(core_sb_full)
            options.SetOutputFile(core_sb_full_spec)
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreFull)
            error = process.SaveCore(options)
            self.assertTrue(error.Success())
            self.assertTrue(os.path.isfile(core_sb_full))
            self.verify_core_file(
                core_sb_full,
                expected_pid,
                expected_modules,
                expected_threads,
                stacks_to_sp_map,
                stacks_to_registers_map,
            )

            self.assertSuccess(process.Kill())
        finally:
            # Clean up the mini dump file.
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(core_stack):
                os.unlink(core_stack)
            if os.path.isfile(core_dirty):
                os.unlink(core_dirty)
            if os.path.isfile(core_full):
                os.unlink(core_full)
            if os.path.isfile(core_sb_stack):
                os.unlink(core_sb_stack)
            if os.path.isfile(core_sb_dirty):
                os.unlink(core_sb_dirty)
            if os.path.isfile(core_sb_full):
                os.unlink(core_sb_full)

    @skipUnlessArch("x86_64")
    @skipUnlessPlatform(["linux"])
    def test_save_linux_mini_dump_thread_options(self):
        """Test that we can save a Linux mini dump
        with a subset of threads"""

        self.build()
        exe = self.getBuildArtifact("a.out")
        thread_subset_dmp = self.getBuildArtifact("core.thread.subset.dmp")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)

            thread_to_include = process.GetThreadAtIndex(0)
            options = lldb.SBSaveCoreOptions()
            thread_subset_spec = lldb.SBFileSpec(thread_subset_dmp)
            options.AddThread(thread_to_include)
            options.SetOutputFile(thread_subset_spec)
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreStackOnly)
            error = process.SaveCore(options)
            self.assertTrue(error.Success())

            core_target = self.dbg.CreateTarget(None)
            core_process = core_target.LoadCore(thread_subset_dmp)

            self.assertTrue(core_process, PROCESS_IS_VALID)
            self.assertEqual(core_process.GetNumThreads(), 1)
            saved_thread = core_process.GetThreadAtIndex(0)
            expected_thread = process.GetThreadAtIndex(0)
            self.assertEqual(expected_thread.GetThreadID(), saved_thread.GetThreadID())
            expected_sp = expected_thread.GetFrameAtIndex(0).GetSP()
            saved_sp = saved_thread.GetFrameAtIndex(0).GetSP()
            self.assertEqual(expected_sp, saved_sp)
            expected_region = lldb.SBMemoryRegionInfo()
            saved_region = lldb.SBMemoryRegionInfo()
            error = core_process.GetMemoryRegionInfo(saved_sp, saved_region)
            self.assertTrue(error.Success(), error.GetCString())
            error = process.GetMemoryRegionInfo(expected_sp, expected_region)
            self.assertTrue(error.Success(), error.GetCString())
            self.assertEqual(
                expected_region.GetRegionBase(), saved_region.GetRegionBase()
            )
            self.assertEqual(
                expected_region.GetRegionEnd(), saved_region.GetRegionEnd()
            )

        finally:
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(thread_subset_dmp):
                os.unlink(thread_subset_dmp)

    @skipUnlessArch("x86_64")
    @skipUnlessPlatform(["linux"])
    def test_save_linux_mini_dump_default_options(self):
        """Test that we can save a Linux mini dump with default SBSaveCoreOptions"""

        self.build()
        exe = self.getBuildArtifact("a.out")
        default_value_file = self.getBuildArtifact("core.defaults.dmp")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)

            process_info = process.GetProcessInfo()
            expected_pid = process_info.GetProcessID() if process_info.IsValid() else -1
            expected_modules = target.modules
            expected_threads = []
            stacks_to_sp_map = {}
            expected_pid = process.GetProcessInfo().GetProcessID()
            stacks_to_registers_map = {}

            for thread_idx in range(process.GetNumThreads()):
                thread = process.GetThreadAtIndex(thread_idx)
                thread_id = thread.GetThreadID()
                expected_threads.append(thread_id)
                stacks_to_sp_map[thread_id] = thread.GetFrameAtIndex(0).GetSP()
                stacks_to_registers_map[thread_id] = thread.GetFrameAtIndex(
                    0
                ).GetRegisters()

            # This is almost identical to the single thread test case because
            # minidump defaults to stacks only, so we want to see if the
            # default options work as expected.
            options = lldb.SBSaveCoreOptions()
            default_value_spec = lldb.SBFileSpec(default_value_file)
            options.SetOutputFile(default_value_spec)
            options.SetPluginName("minidump")
            error = process.SaveCore(options)
            self.assertTrue(error.Success())

            self.verify_core_file(
                default_value_file,
                expected_pid,
                expected_modules,
                expected_threads,
                stacks_to_sp_map,
                stacks_to_registers_map,
            )

        finally:
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(default_value_file):
                os.unlink(default_value_file)

    @skipUnlessArch("x86_64")
    @skipUnlessPlatform(["linux"])
    def test_save_linux_minidump_one_region(self):
        """Test that we can save a Linux mini dump with one region in sbsavecore regions"""

        self.build()
        exe = self.getBuildArtifact("a.out")
        one_region_file = self.getBuildArtifact("core.one_region.dmp")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)

            memory_region = lldb.SBMemoryRegionInfo()
            memory_list = process.GetMemoryRegions()
            memory_list.GetMemoryRegionAtIndex(0, memory_region)

            # This is almost identical to the single thread test case because
            # minidump defaults to stacks only, so we want to see if the
            # default options work as expected.
            options = lldb.SBSaveCoreOptions()
            file_spec = lldb.SBFileSpec(one_region_file)
            options.SetOutputFile(file_spec)
            options.SetPluginName("minidump")
            options.AddMemoryRegionToSave(memory_region)
            options.SetStyle(lldb.eSaveCoreCustomOnly)
            error = process.SaveCore(options)
            print(f"Error: {error.GetCString()}")
            self.assertTrue(error.Success(), error.GetCString())

            core_target = self.dbg.CreateTarget(None)
            core_proc = core_target.LoadCore(one_region_file)
            core_memory_list = core_proc.GetMemoryRegions()
            # Note because the /proc/pid maps are included on linux, we can't
            # depend on size for validation, so we'll ensure the first region
            # is present and then assert we fail on the second.
            core_memory_region = lldb.SBMemoryRegionInfo()
            core_memory_list.GetMemoryRegionAtIndex(0, core_memory_region)
            self.assertEqual(
                core_memory_region.GetRegionBase(), memory_region.GetRegionBase()
            )
            self.assertEqual(
                core_memory_region.GetRegionEnd(), memory_region.GetRegionEnd()
            )

            region_two = lldb.SBMemoryRegionInfo()
            core_memory_list.GetMemoryRegionAtIndex(1, region_two)
            err = lldb.SBError()
            content = core_proc.ReadMemory(region_two.GetRegionBase(), 1, err)
            self.assertTrue(err.Fail(), "Should fail to read memory")

        finally:
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(one_region_file):
                os.unlink(one_region_file)

    @skipUnlessArch("x86_64")
    @skipUnlessPlatform(["linux"])
    def test_save_minidump_custom_save_style(self):
        """Test that verifies a custom and unspecified save style fails for
        containing no data to save"""

        self.build()
        exe = self.getBuildArtifact("a.out")
        custom_file = self.getBuildArtifact("core.custom.dmp")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)

            options = lldb.SBSaveCoreOptions()
            options.SetOutputFile(lldb.SBFileSpec(custom_file))
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreCustomOnly)

            error = process.SaveCore(options)
            self.assertTrue(error.Fail())
            self.assertEqual(
                error.GetCString(), "no valid address ranges found for core style"
            )

        finally:
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(custom_file):
                os.unlink(custom_file)

    def save_core_with_region(self, process, region_index):
        try:
            custom_file = self.getBuildArtifact("core.custom.dmp")
            memory_region = lldb.SBMemoryRegionInfo()
            memory_list = process.GetMemoryRegions()
            memory_list.GetMemoryRegionAtIndex(0, memory_region)
            options = lldb.SBSaveCoreOptions()
            options.SetOutputFile(lldb.SBFileSpec(custom_file))
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreFull)

            error = process.SaveCore(options)
            self.assertTrue(error.Success())
            core_target = self.dbg.CreateTarget(None)
            core_proc = core_target.LoadCore(custom_file)
            core_memory_list = core_proc.GetMemoryRegions()
            # proc/pid/ maps are included on linux, so we can't depend on size
            # for validation, we make a set of all the ranges,
            # and ensure no duplicates!
            range_set = set()
            for x in range(core_memory_list.GetSize()):
                core_memory_region = lldb.SBMemoryRegionInfo()
                core_memory_list.GetMemoryRegionAtIndex(x, core_memory_region)
                mem_tuple = (
                    core_memory_region.GetRegionBase(),
                    core_memory_region.GetRegionEnd(),
                )
                self.assertTrue(
                    mem_tuple not in range_set, "Duplicate memory region found"
                )
                range_set.add(mem_tuple)
        finally:
            if os.path.isfile(custom_file):
                os.unlink(custom_file)

    @skipUnlessArch("x86_64")
    @skipUnlessPlatform(["linux"])
    def test_save_minidump_custom_save_style_duplicated_regions(self):
        """Test that verifies a custom and unspecified save style fails for
        containing no data to save"""

        self.build()
        exe = self.getBuildArtifact("a.out")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)

            memory_list = process.GetMemoryRegions()
            # Test that we don't duplicate regions, by duplicating regions
            # at various indices.
            self.save_core_with_region(process, 0)
            self.save_core_with_region(process, len(memory_list) - 1)

        finally:
            self.assertTrue(self.dbg.DeleteTarget(target))

    @skipUnlessPlatform(["linux"])
    def minidump_deleted_on_save_failure(self):
        """Test that verifies the minidump file is deleted after an error"""

        self.build()
        exe = self.getBuildArtifact("a.out")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)

            custom_file = self.getBuildArtifact("core.should.be.deleted.custom.dmp")
            options = lldb.SBSaveCoreOptions()
            options.SetOutputFile(lldb.SBFileSpec(custom_file))
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreCustomOnly)
            # We set custom only and have no thread list and have no memory.
            error = process.SaveCore(options)
            self.assertTrue(error.Fail())
            self.assertIn(
                "no valid address ranges found for core style", error.GetCString()
            )
            self.assertTrue(not os.path.isfile(custom_file))

        finally:
            self.assertTrue(self.dbg.DeleteTarget(target))

    @skipUnlessPlatform(["linux"])
    @skipUnlessArch("x86_64")
    def minidump_saves_fs_base_region(self):
        """Test that verifies the minidump file saves region for fs_base"""

        self.build()
        exe = self.getBuildArtifact("a.out")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)
            thread = process.GetThreadAtIndex(0)
            custom_file = self.getBuildArtifact("core.reg_region.dmp")
            options = lldb.SBSaveCoreOptions()
            options.SetOutputFile(lldb.SBFileSpec(custom_file))
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreCustomOnly)
            options.AddThread(thread)
            error = process.SaveCore(options)
            self.assertTrue(error.Success())

            registers = thread.GetFrameAtIndex(0).GetRegisters()
            fs_base = registers.GetFirstValueByName("fs_base").GetValueAsUnsigned()
            self.assertTrue(fs_base != 0)
            core_target = self.dbg.CreateTarget(None)
            core_proc = core_target.LoadCore(one_region_file)
            core_region_list = core_proc.GetMemoryRegions()
            live_region_list = process.GetMemoryRegions()
            live_region = lldb.SBMemoryRegionInfo()
            live_region_list.GetMemoryRegionForAddress(fs_base, live_region)
            core_region = lldb.SBMemoryRegionInfo()
            error = core_region_list.GetMemoryRegionForAddress(fs_base, core_region)
            self.assertTrue(error.Success())
            self.assertEqual(live_region, core_region)

        finally:
            self.assertTrue(self.dbg.DeleteTarget(target))
            self.assertTrue(self.dbg.DeleteTarget(core_target))
            if os.path.isfile(custom_file):
                os.unlink(custom_file)

    def minidump_deterministic_difference(self):
        """Test that verifies that two minidumps produced are identical."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )

            core_styles = [
                lldb.eSaveCoreStackOnly,
                lldb.eSaveCoreDirtyOnly,
                lldb.eSaveCoreFull,
            ]
            for style in core_styles:
                spec_one = lldb.SBFileSpec(self.getBuildArtifact("core.one.dmp"))
                spec_two = lldb.SBFileSpec(self.getBuildArtifact("core.two.dmp"))
                options = lldb.SBSaveCoreOptions()
                options.SetOutputFile(spec_one)
                options.SetPluginName("minidump")
                options.SetStyle(style)
                error = process.SaveCore(options)
                self.assertTrue(error.Success())
                options.SetOutputFile(spec_two)
                error = process.SaveCore(options)
                self.assertTrue(error.Success())

                file_one = None
                file_two = None
                with open(spec_one.GetFileName(), mode="rb") as file:
                    file_one = file.read()
                with open(spec_two.GetFileName(), mode="rb") as file:
                    file_two = file.read()
                self.assertEqual(file_one, file_two)
                self.assertTrue(os.unlink(spec_one.GetFileName()))
                self.assertTrue(os.unlink(spec_two.GetFileName()))
        finally:
            self.assertTrue(self.dbg.DeleteTarget(target))

    @skipUnlessPlatform(["linux"])
    @skipUnlessArch("x86_64")
    def minidump_saves_tls(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)
            thread = process.GetThreadAtIndex(0)
            tls_file = self.getBuildArtifact("core.tls.dmp")
            options = lldb.SBSaveCoreOptions()
            options.SetOutputFile(lldb.SBFileSpec(tls_file))
            options.SetPluginName("minidump")
            options.SetStyle(lldb.eSaveCoreCustomOnly)
            options.AddThread(thread)
            error = process.SaveCore(options)
            self.assertTrue(error.Success())
            core_target = self.dbg.CreateTarget(None)
            core_proc = core_target.LoadCore(tls_file)
            frame = core_proc.GetThreadAtIndex(0).GetFrameAtIndex(0)
            tls_val = frame.FindValue("lf")
            self.assertEqual(tls_val.GetValueAsUnsigned(), 42)

        except:
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(tls_file):
                os.unlink(tls_file)

    @skipUnlessPlatform(["linux"])
    @skipUnlessArch("x86_64")
    def test_invalid_custom_regions_not_included(self):
        options = lldb.SBSaveCoreOptions()
        self.build()
        exe = self.getBuildArtifact("a.out")
        output_file = self.getBuildArtifact("no_empty_regions.dmp")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)
            options.SetPluginName("minidump")
            options.SetOutputFile(lldb.SBFileSpec(output_file))
            options.SetStyle(lldb.eSaveCoreCustomOnly)
            region_one = lldb.SBMemoryRegionInfo()
            process.GetMemoryRegions().GetMemoryRegionAtIndex(0, region_one)
            options.AddMemoryRegionToSave(region_one)
            empty_region = lldb.SBMemoryRegionInfo(
                "empty region", 0x0, 0x0, 3, True, False
            )
            options.AddMemoryRegionToSave(empty_region)
            region_with_no_permissions = lldb.SBMemoryRegionInfo(
                "no permissions", 0x2AAA, 0x2BBB, 0, True, False
            )
            options.AddMemoryRegionToSave(region_with_no_permissions)
            error = process.SaveCore(options)
            self.assertTrue(error.Success(), error.GetCString())
            core_target = self.dbg.CreateTarget(None)
            core_process = core_target.LoadCore(output_file)
            self.assertNotIn(
                region_with_no_permissions, core_process.GetMemoryRegions()
            )
            self.assertNotIn(empty_region, core_process.GetMemoryRegions())
        finally:
            if os.path.isfile(output_file):
                os.unlink(output_file)



"""
Make sure the stop reason of a thread that did not run
during an expression is not changed by running the expression
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestStopReasonAfterExpression(TestBase):
    @skipIfWindows
    @expectedFailureAll(oslist=["freebsd"], bugnumber="llvm.org/pr48415")
    @expectedFlakeyNetBSD
    def test_thread_state_after_expr(self):
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        self.do_test()

    def do_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        self.assertEqual(bkpt.GetNumLocations(), 2, "Got two locations")

        # So now thread holds the main thread.  Continue to hit the
        # breakpoint again on the spawned thread:

        threads = lldbutil.continue_to_breakpoint(process, bkpt)
        self.assertEqual(len(threads), 1, "Hit the breakpoint the second time")
        other_thread = threads[0]

        self.assertNotEqual(
            thread.GetThreadID(), other_thread.GetThreadID(), "A different thread"
        )
        # Run an expression ONLY on other_thread.  Don't let thread run:
        options = lldb.SBExpressionOptions()
        options.SetTryAllThreads(False)
        options.SetStopOthers(True)

        result = thread.frames[0].EvaluateExpression(
            '(int) printf("Hello\\n")', options
        )
        self.assertSuccess(result.GetError(), "Expression failed")

        stop_reason = other_thread.GetStopReason()

        self.assertStopReason(
            stop_reason,
            lldb.eStopReasonBreakpoint,
            "Still records stopped at breakpoint: %s"
            % (lldbutil.stop_reason_to_str(stop_reason)),
        )
        self.assertEqual(
            other_thread.GetStopReasonDataAtIndex(0),
            1,
            "Still records stopped at right breakpoint",
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @skipUnlessDarwin
    def test_objc_self(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "check self", lldb.SBFileSpec("main.mm")
        )
        self.expect("frame variable _ivar", startstr="(int) _ivar = 30")

    @skipUnlessDarwin
    def test_objc_explicit_self(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "check explicit self", lldb.SBFileSpec("main.mm")
        )
        self.expect("frame variable _ivar", startstr="(int) _ivar = 30")

    @skipUnlessDarwin
    def test_cpp_this(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "check this", lldb.SBFileSpec("main.mm")
        )
        self.expect("frame variable m_field", startstr="(int) m_field = 41")



"""
Test that ASan memory history provider returns correct stack traces
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbplatform
from lldbsuite.test import lldbutil
from lldbsuite.test_event.build_exception import BuildError


class MemoryHistoryTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    @skipIfFreeBSD  # llvm.org/pr21136 runtimes not yet available by default
    @expectedFailureNetBSD
    @skipUnlessAddressSanitizer
    def test_compiler_rt_asan(self):
        self.build(make_targets=["compiler_rt-asan"])
        self.compiler_rt_asan_tests()

    @skipUnlessDarwin
    @skipIf(bugnumber="rdar://109913184&143590169")
    def test_libsanitizers_asan(self):
        try:
            self.build(make_targets=["libsanitizers-asan"])
        except BuildError as e:
            self.skipTest("failed to build with libsanitizers")
        self.libsanitizers_asan_tests()

    @skipUnlessDarwin
    @skipIf(macos_version=["<", "15.5"])
    def test_libsanitizers_traces(self):
        self.build(make_targets=["libsanitizers-traces"])
        self.libsanitizers_traces_tests()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.line_malloc = line_number("main.c", "// malloc line")
        self.line_malloc2 = line_number("main.c", "// malloc2 line")
        self.line_free = line_number("main.c", "// free line")
        self.line_breakpoint = line_number("main.c", "// break line")

    def check_traces(self):
        self.expect(
            "memory history 'pointer'",
            substrs=[
                "Memory deallocated by Thread",
                "a.out`f2",
                f"main.c:{self.line_free}",
                "Memory allocated by Thread",
                "a.out`f1",
                f"main.c:{self.line_malloc}",
            ],
        )

    # Set breakpoint: after free, but before bug
    def set_breakpoint(self, target):
        bkpt = target.BreakpointCreateByLocation("main.c", self.line_breakpoint)
        self.assertGreater(bkpt.GetNumLocations(), 0, "Set the breakpoint successfully")

    def run_to_breakpoint(self, target):
        self.set_breakpoint(target)
        self.runCmd("run")
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

    def libsanitizers_traces_tests(self):
        target = self.createTestTarget()

        self.runCmd("env SanitizersAllocationTraces=all")

        self.run_to_breakpoint(target)
        self.check_traces()

    def libsanitizers_asan_tests(self):
        target = self.createTestTarget()

        self.runCmd("env SanitizersAddress=1 MallocSanitizerZone=1")

        self.run_to_breakpoint(target)
        self.check_traces()

        self.runCmd("continue")

        # Stop on report
        self.expect(
            "thread list",
            "Process should be stopped due to ASan report",
            substrs=["stopped", "stop reason = Use of deallocated memory"],
        )
        self.check_traces()

        if self.platformIsDarwin():
            # Make sure we're not stopped in the sanitizer library but instead at the
            # point of failure in the user-code.
            self.assertEqual(self.frame().GetFunctionName(), "main")

        # do the same using SB API
        process = self.dbg.GetSelectedTarget().process
        val = (
            process.GetSelectedThread().GetSelectedFrame().EvaluateExpression("pointer")
        )
        addr = val.GetValueAsUnsigned()
        threads = process.GetHistoryThreads(addr)
        self.assertEqual(threads.GetSize(), 2)

        history_thread = threads.GetThreadAtIndex(0)
        self.assertTrue(history_thread.num_frames >= 2)
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetFileSpec().GetFilename(),
            "main.c",
        )

        history_thread = threads.GetThreadAtIndex(1)
        self.assertTrue(history_thread.num_frames >= 2)
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetFileSpec().GetFilename(),
            "main.c",
        )

        # let's free the container (SBThreadCollection) and see if the
        # SBThreads still live
        threads = None
        self.assertTrue(history_thread.num_frames >= 2)
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetFileSpec().GetFilename(),
            "main.c",
        )

    def compiler_rt_asan_tests(self):
        target = self.createTestTarget()

        self.registerSanitizerLibrariesWithTarget(target)

        self.set_breakpoint(target)

        # "memory history" command should not work without a process
        self.expect(
            "memory history 0",
            error=True,
            substrs=["Command requires a current process"],
        )

        self.runCmd("run")

        stop_reason = (
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason()
        )
        if stop_reason == lldb.eStopReasonExec:
            # On OS X 10.10 and older, we need to re-exec to enable
            # interceptors.
            self.runCmd("continue")

        # the stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # test that the ASan dylib is present
        self.expect(
            "image lookup -n __asan_describe_address",
            "__asan_describe_address should be present",
            substrs=["1 match found"],
        )

        self.check_traces()

        # do the same using SB API
        process = self.dbg.GetSelectedTarget().process
        val = (
            process.GetSelectedThread().GetSelectedFrame().EvaluateExpression("pointer")
        )
        addr = val.GetValueAsUnsigned()
        threads = process.GetHistoryThreads(addr)
        self.assertEqual(threads.GetSize(), 2)

        history_thread = threads.GetThreadAtIndex(0)
        self.assertGreaterEqual(history_thread.num_frames, 2)
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetFileSpec().GetFilename(),
            "main.c",
        )
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetLine(), self.line_free
        )

        history_thread = threads.GetThreadAtIndex(1)
        self.assertGreaterEqual(history_thread.num_frames, 2)
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetFileSpec().GetFilename(),
            "main.c",
        )
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetLine(), self.line_malloc
        )

        # let's free the container (SBThreadCollection) and see if the
        # SBThreads still live
        threads = None
        self.assertGreaterEqual(history_thread.num_frames, 2)
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetFileSpec().GetFilename(),
            "main.c",
        )
        self.assertEqual(
            history_thread.frames[1].GetLineEntry().GetLine(), self.line_malloc
        )

        # ASan will break when a report occurs and we'll try the API then
        self.runCmd("continue")

        self.expect(
            "thread list",
            "Process should be stopped due to ASan report",
            substrs=["stopped", "stop reason = Use of deallocated memory"],
        )

        self.check_traces()

        if self.platformIsDarwin():
            # Make sure we're not stopped in the sanitizer library but instead at the
            # point of failure in the user-code.
            self.assertEqual(self.frame().GetFunctionName(), "main")

        # make sure the 'memory history' command still works even when we're
        # generating a report now
        self.expect(
            "memory history 'another_pointer'",
            substrs=[
                "Memory allocated by Thread",
                "a.out`f1",
                "main.c:%d" % self.line_malloc2,
            ],
        )



"""
Test lldb-dap launch request.
"""

import lldbdap_testcase
import os
import tempfile


class TestDAP_launch_no_lldbinit_flag(lldbdap_testcase.DAPTestCaseBase):
    """
    Test that the --no-lldbinit flag prevents sourcing .lldbinit files.
    """

    def test(self):
        # Create a temporary .lldbinit file in the home directory
        with tempfile.TemporaryDirectory() as temp_home:
            lldbinit_path = os.path.join(temp_home, ".lldbinit")

            # Write a command to the .lldbinit file that would set a unique setting
            with open(lldbinit_path, "w") as f:
                f.write("settings set stop-disassembly-display never\n")
                f.write("settings set target.x86-disassembly-flavor intel\n")

            # Test with --no-lldbinit flag (should NOT source .lldbinit)
            self.build_and_create_debug_adapter(
                lldbDAPEnv={"HOME": temp_home}, additional_args=["--no-lldbinit"]
            )
            program = self.getBuildArtifact("a.out")

            # Use initCommands to check if .lldbinit was sourced
            initCommands = ["settings show stop-disassembly-display"]

            # Launch with initCommands to check the setting
            self.launch(program, initCommands=initCommands)
            self.continue_to_exit()

            # Get console output to verify the setting was NOT set from .lldbinit
            output = self.get_console()
            self.assertTrue(output and len(output) > 0, "expect console output")

            # Verify the setting has default value, not "never" from .lldbinit
            self.assertNotIn(
                "never",
                output,
                "Setting should have default value when --no-lldbinit is used",
            )

            # Verify the initCommands were executed
            self.verify_commands("initCommands", output, initCommands)



"""
Test frame providers for scripted frame provider functionality.

These providers demonstrate various merge strategies:
- Replace: Replace entire stack
- Prepend: Add frames before real stack
- Append: Add frames after real stack

It also shows the ability to mix a dictionary, a ScriptedFrame or an SBFrame
index to create stackframes
"""

import os
import lldb
from lldb.plugins.scripted_process import ScriptedFrame
from lldb.plugins.scripted_frame_provider import ScriptedFrameProvider


class ReplaceFrameProvider(ScriptedFrameProvider):
    """Replace entire stack with custom frames."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)
        self.frames = [
            {
                "idx": 0,
                "pc": 0x1000,
            },
            0,
            {
                "idx": 2,
                "pc": 0x3000,
            },
        ]

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Replace entire stack with 3 custom frames"

    def get_frame_at_index(self, index):
        if index >= len(self.frames):
            return None
        return self.frames[index]


class PrependFrameProvider(ScriptedFrameProvider):
    """Prepend synthetic frames before real stack."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Prepend 2 synthetic frames before real stack"

    def get_frame_at_index(self, index):
        if index == 0:
            return {"pc": 0x9000}
        elif index == 1:
            return {"pc": 0xA000}
        elif index - 2 < len(self.input_frames):
            return index - 2  # Return real frame index.
        return None


class AppendFrameProvider(ScriptedFrameProvider):
    """Append synthetic frames after real stack."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Append 1 synthetic frame after real stack"

    def get_frame_at_index(self, index):
        if index < len(self.input_frames):
            return index  # Return real frame index.
        elif index == len(self.input_frames):
            return {
                "idx": 1,
                "pc": 0x10,
            }
        return None


class CustomScriptedFrame(ScriptedFrame):
    """Custom scripted frame with full control over frame behavior."""

    def __init__(self, thread, idx, pc, function_name):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.pc = pc
        self.function_name = function_name

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """Return the program counter."""
        return self.pc

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def is_artificial(self):
        """Mark as artificial frame."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        """No register context for this test."""
        return None


class ScriptedFrameObjectProvider(ScriptedFrameProvider):
    """Provider that returns ScriptedFrame objects instead of dictionaries."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider returning custom ScriptedFrame objects"

    def get_frame_at_index(self, index):
        """Return ScriptedFrame objects or dictionaries based on index."""
        if index == 0:
            return CustomScriptedFrame(
                self.thread, 0, 0x5000, "custom_scripted_frame_0"
            )
        elif index == 1:
            return {"pc": 0x6000}
        elif index == 2:
            return CustomScriptedFrame(
                self.thread, 2, 0x7000, "custom_scripted_frame_2"
            )
        elif index == 3:
            return len(self.input_frames) - 2  # Real frame index.
        elif index == 4:
            return len(self.input_frames) - 1  # Real frame index.
        return None


class ThreadFilterFrameProvider(ScriptedFrameProvider):
    """Provider that only applies to thread with ID 1."""

    @staticmethod
    def applies_to_thread(thread):
        """Only apply to thread with index ID 1."""
        return thread.GetIndexID() == 1

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that only applies to thread ID 1"

    def get_frame_at_index(self, index):
        """Return a single synthetic frame."""
        if index == 0:
            return {"pc": 0xFFFF}
        return None


class CircularDependencyTestProvider(ScriptedFrameProvider):
    """
    Provider that tests the circular dependency fix.

    This provider accesses input_frames during __init__ and calls methods
    on those frames. Before the fix, this would cause a circular dependency:
    - Thread::GetStackFrameList() creates provider
    - Provider's __init__ accesses input_frames[0]
    - SBFrame::GetPC() tries to resolve ExecutionContextRef
    - ExecutionContextRef::GetFrameSP() calls Thread::GetStackFrameList()
    - Re-enters initialization -> circular dependency!

    With the fix, ExecutionContextRef remembers the frame list, so it doesn't
    re-enter Thread::GetStackFrameList().
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

        # This would cause circular dependency before the fix!
        # Accessing frames and calling methods on them during __init__
        self.original_frame_count = len(input_frames)
        self.original_pcs = []

        # Call GetPC() on each input frame - this triggers ExecutionContextRef resolution.
        for i in range(min(3, len(input_frames))):
            frame = input_frames[i]
            if frame.IsValid():
                pc = frame.GetPC()
                self.original_pcs.append(pc)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that tests circular dependency fix by accessing frames in __init__"

    def get_frame_at_index(self, index):
        """Prepend a synthetic frame, then pass through original frames."""
        if index == 0:
            # Synthetic frame at index 0.
            return {"pc": 0xDEADBEEF}
        elif index - 1 < self.original_frame_count:
            # Pass through original frames at indices 1, 2, 3, ...
            return index - 1
        return None


class PythonSourceFrame(ScriptedFrame):
    """Scripted frame that points to Python source code."""

    def __init__(self, thread, idx, function_name, python_file, line_number):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.function_name = function_name
        self.python_file = python_file
        self.line_number = line_number

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """PC-less frame - return invalid address."""
        return lldb.LLDB_INVALID_ADDRESS

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def get_symbol_context(self):
        """Return a symbol context with LineEntry pointing to Python source."""
        # Create a LineEntry pointing to the Python source file
        line_entry = lldb.SBLineEntry()
        line_entry.SetFileSpec(lldb.SBFileSpec(self.python_file, True))
        line_entry.SetLine(self.line_number)
        line_entry.SetColumn(0)

        # Create a symbol context with the line entry
        sym_ctx = lldb.SBSymbolContext()
        sym_ctx.SetLineEntry(line_entry)

        return sym_ctx

    def is_artificial(self):
        """Not artificial."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        """No register context for PC-less frames."""
        return None


class PythonSourceFrameProvider(ScriptedFrameProvider):
    """
    Provider that demonstrates Python source display in scripted frames.

    This provider prepends frames pointing to Python source code, showing
    that PC-less frames can display Python source files with proper line
    numbers and module/compile unit information.
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

        # Find the python_helper.py file
        current_dir = os.path.dirname(os.path.abspath(__file__))
        self.python_file = os.path.join(current_dir, "python_helper.py")

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that prepends frames pointing to Python source"

    def get_frame_at_index(self, index):
        """Return Python source frames followed by original frames."""
        if index == 0:
            # Frame pointing to compute_fibonacci function (line 7)
            return PythonSourceFrame(
                self.thread, 0, "compute_fibonacci", self.python_file, 7
            )
        elif index == 1:
            # Frame pointing to process_data function (line 16)
            return PythonSourceFrame(
                self.thread, 1, "process_data", self.python_file, 16
            )
        elif index == 2:
            # Frame pointing to main function (line 27)
            return PythonSourceFrame(self.thread, 2, "main", self.python_file, 27)
        elif index - 3 < len(self.input_frames):
            # Pass through original frames
            return index - 3
        return None


class ValidPCNoModuleFrame(ScriptedFrame):
    """Scripted frame with a valid PC but no associated module."""

    def __init__(self, thread, idx, pc, function_name):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.pc = pc
        self.function_name = function_name

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """Return the program counter."""
        return self.pc

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def is_artificial(self):
        """Not artificial."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        """No register context."""
        return None


class ValidPCNoModuleFrameProvider(ScriptedFrameProvider):
    """
    Provider that demonstrates frames with valid PC but no module.

    This tests that backtrace output handles frames that have a valid
    program counter but cannot be resolved to any loaded module.
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that prepends frames with valid PC but no module"

    def get_frame_at_index(self, index):
        """Return frames with valid PCs but no module information."""
        if index == 0:
            # Frame with valid PC (0x1234000) but no module
            return ValidPCNoModuleFrame(self.thread, 0, 0x1234000, "unknown_function_1")
        elif index == 1:
            # Another frame with valid PC (0x5678000) but no module
            return ValidPCNoModuleFrame(self.thread, 1, 0x5678000, "unknown_function_2")
        elif index - 2 < len(self.input_frames):
            # Pass through original frames
            return index - 2
        return None


class AddFooFrameProvider(ScriptedFrameProvider):
    """Add a single 'foo' frame at the beginning."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Add 'foo' frame at beginning"

    @staticmethod
    def get_priority():
        """Return priority 10 (runs first in chain)."""
        return 10

    def get_frame_at_index(self, index):
        if index == 0:
            # Return synthetic "foo" frame
            return CustomScriptedFrame(self.thread, 0, 0xF00, "foo")
        elif index - 1 < len(self.input_frames):
            # Pass through input frames (shifted by 1)
            return index - 1
        return None


class AddBarFrameProvider(ScriptedFrameProvider):
    """Add a single 'bar' frame at the beginning."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Add 'bar' frame at beginning"

    @staticmethod
    def get_priority():
        """Return priority 20 (runs second in chain)."""
        return 20

    def get_frame_at_index(self, index):
        if index == 0:
            # Return synthetic "bar" frame
            return CustomScriptedFrame(self.thread, 0, 0xBAA, "bar")
        elif index - 1 < len(self.input_frames):
            # Pass through input frames (shifted by 1)
            return index - 1
        return None


class AddBazFrameProvider(ScriptedFrameProvider):
    """Add a single 'baz' frame at the beginning."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Add 'baz' frame at beginning"

    @staticmethod
    def get_priority():
        """Return priority 30 (runs last in chain)."""
        return 30

    def get_frame_at_index(self, index):
        if index == 0:
            # Return synthetic "baz" frame
            return CustomScriptedFrame(self.thread, 0, 0xBAC, "baz")
        elif index - 1 < len(self.input_frames):
            # Pass through input frames (shifted by 1)
            return index - 1
        return None


class ThreadValidatingFrame(ScriptedFrame):
    """Frame that validates thread member is correctly set after init.

    This catches the regression where ScriptedFrame.__init__ used
    GetThreadByIndexID(tid) instead of GetThreadByID(tid). The thread ID
    and index ID are different, so using the wrong API produces an invalid
    thread. The validation result is encoded in the function name so the
    test can verify it through the SB API.
    """

    def __init__(self, thread, idx):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)
        self.idx = idx
        # Store validation results after base class init
        self.thread_valid = self.thread is not None and self.thread.IsValid()
        self.thread_id = self.thread.GetThreadID() if self.thread_valid else 0

    def get_id(self):
        return self.idx

    def get_function_name(self):
        # Encode validation in function name so test can verify via SB API
        if self.thread_valid:
            return f"thread_valid_id_{self.thread_id:#x}"
        return "thread_INVALID"

    def is_artificial(self):
        return False

    def is_hidden(self):
        return False

    def get_register_context(self):
        return None


class ThreadValidatingFrameProvider(ScriptedFrameProvider):
    """Provider that validates ScriptedFrame.thread member initialization.

    Uses ThreadValidatingFrame to verify that self.thread is correctly set
    via GetThreadByID (not GetThreadByIndexID) after ScriptedFrame.__init__.
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        return "Validate ScriptedFrame.thread member initialization"

    def get_frame_at_index(self, index):
        if index == 0:
            return ThreadValidatingFrame(self.thread, 0)
        elif index - 1 < len(self.input_frames):
            return index - 1
        return None


class ValueProvidingFrame(ScriptedFrame):
    """Scripted frame with a valid PC but no associated module."""

    def __init__(self, thread, idx, pc, function_name, variable):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.pc = pc
        self.function_name = function_name
        self.variable = variable

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """Return the program counter."""
        return self.pc

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def is_artificial(self):
        """Not artificial."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        """No register context."""
        return None

    def get_variables(self):
        """"""
        out = lldb.SBValueList()
        out.Append(self.variable)
        return out

    def get_value_for_variable_expression(self, expr, options, error: lldb.SBError):
        out = lldb.SBValue()
        if expr == "_handler_one":
            out = self.variable.CreateValueFromExpression("_handler_one", "(uint32_t)1")
        elif self.variable.name in expr:
            out = self.variable.CreateValueFromExpression("_expr", expr)

        if out.IsValid():
            return out

        error.SetErrorString(f"expression {expr} failed")
        return None


class ValueProvidingFrameProvider(ScriptedFrameProvider):
    """Add a single 'value-provider' frame at the beginning."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Add 'value-provider' frame at beginning"

    def get_frame_at_index(self, index):
        if index == 0:
            f = self.input_frames.GetFrameAtIndex(index)
            # Find some variable we can give to the frame.
            variable = f.FindVariable("variable_in_main")
            # Return synthetic "value-provider" frame
            return ValueProvidingFrame(
                self.thread, 0, 0xF00, "value-provider", variable
            )
        elif index - 1 < len(self.input_frames):
            # Pass through input frames (shifted by 1)
            return index - 1
        return None



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class MyResponder(MockGDBServerResponder):
    def qXferRead(self, obj, annex, offset, length):
        if annex == "target.xml":
            return (
                """<?xml version="1.0"?>
                <target version="1.0">
                  <architecture>i386:x86-64</architecture>
                  <feature name="org.gnu.gdb.i386.core">

                 <flags id="i386_eflags" size="4">
                 <field name="CF" start="0" end="0"/>
                 <field name="" start="1" end="1"/>
                 <field name="PF" start="2" end="2"/>
                 <field name="AF" start="4" end="4"/>
                 <field name="ZF" start="6" end="6"/>
                 <field name="SF" start="7" end="7"/>
                 <field name="TF" start="8" end="8"/>
                 <field name="IF" start="9" end="9"/>
                 <field name="DF" start="10" end="10"/>
                 <field name="OF" start="11" end="11"/>
                 <field name="NT" start="14" end="14"/>
                 <field name="RF" start="16" end="16"/>
                 <field name="VM" start="17" end="17"/>
                 <field name="AC" start="18" end="18"/>
                 <field name="VIF" start="19" end="19"/>
                 <field name="VIP" start="20" end="20"/>
                 <field name="ID" start="21" end="21"/>
                 </flags>

                    <reg name="rax" bitsize="64" regnum="0" type="int" group="general"/>
                    <reg name="rbx" bitsize="64" regnum="1" type="int" group="general"/>
                    <reg name="rcx" bitsize="64" regnum="2" type="int" group="general"/>
                    <reg name="rdx" bitsize="64" regnum="3" type="int" group="general"/>
                    <reg name="rsi" bitsize="64" regnum="4" type="int" group="general"/>
                    <reg name="rdi" bitsize="64" regnum="5" type="int" group="general"/>
                    <reg name="rbp" bitsize="64" regnum="6" type="data_ptr" group="general"/>
                    <reg name="rsp" bitsize="64" regnum="7" type="data_ptr" group="general"/>
                    <reg name="r8" bitsize="64"  regnum="8" type="int" group="general"/>
                    <reg name="r9" bitsize="64"  regnum="9" type="int" group="general"/>
                    <reg name="r10" bitsize="64" regnum="10" type="int" group="general"/>
                    <reg name="r11" bitsize="64" regnum="11" type="int" group="general"/>
                    <reg name="r12" bitsize="64" regnum="12" type="int" group="general"/>
                    <reg name="r13" bitsize="64" regnum="13" type="int" group="general"/>
                    <reg name="r14" bitsize="64" regnum="14" type="int" group="general"/>
                    <reg name="r15" bitsize="64" regnum="15" type="int" group="general"/>
                    <reg name="rip" bitsize="64" regnum="16" type="code_ptr" group="general"/>
                    <reg name="eflags" bitsize="32" regnum="17" type="i386_eflags" group="general"/>

                    <reg name="cs" bitsize="32" regnum="18" type="int" group="general"/>
                    <reg name="ss" bitsize="32" regnum="19" type="int" group="general"/>
                    <reg name="ds" bitsize="32" regnum="20" type="int" group="general"/>
                    <reg name="es" bitsize="32" regnum="21" type="int" group="general"/>
                    <reg name="fs" bitsize="32" regnum="22" type="int" group="general"/>
                    <reg name="gs" bitsize="32" regnum="23" type="int" group="general"/>

                    <reg name="st0" bitsize="80" regnum="24" type="i387_ext" group="float"/>
                    <reg name="st1" bitsize="80" regnum="25" type="i387_ext" group="float"/>
                    <reg name="st2" bitsize="80" regnum="26" type="i387_ext" group="float"/>
                    <reg name="st3" bitsize="80" regnum="27" type="i387_ext" group="float"/>
                    <reg name="st4" bitsize="80" regnum="28" type="i387_ext" group="float"/>
                    <reg name="st5" bitsize="80" regnum="29" type="i387_ext" group="float"/>
                    <reg name="st6" bitsize="80" regnum="30" type="i387_ext" group="float"/>
                    <reg name="st7" bitsize="80" regnum="31" type="i387_ext" group="float"/>

                    <reg name="fctrl" bitsize="32" regnum="32" type="int" group="float"/>
                    <reg name="fstat" bitsize="32" regnum="33" type="int" group="float"/>
                    <reg name="ftag"  bitsize="32" regnum="34" type="int" group="float"/>
                    <reg name="fiseg" bitsize="32" regnum="35" type="int" group="float"/>
                    <reg name="fioff" bitsize="32" regnum="36" type="int" group="float"/>
                    <reg name="foseg" bitsize="32" regnum="37" type="int" group="float"/>
                    <reg name="fooff" bitsize="32" regnum="38" type="int" group="float"/>
                    <reg name="fop"   bitsize="32" regnum="39" type="int" group="float"/>
                  </feature>
                </target>""",
                False,
            )
        else:
            return None, False

    def qC(self):
        return "QC1"

    def haltReason(self):
        return "T05thread:00000001;06:9038d60f00700000;07:98b4062680ffffff;10:c0d7bf1b80ffffff;"

    def readRegister(self, register):
        regs = {
            0x0: "00b0060000610000",
            0xA: "68fe471c80ffffff",
            0xC: "60574a1c80ffffff",
            0xD: "18f3042680ffffff",
            0xE: "be8a4d7142000000",
            0xF: "50df471c80ffffff",
            0x10: "c0d7bf1b80ffffff",
        }
        if register in regs:
            return regs[register]
        else:
            return "0000000000000000"


class TestTargetXMLArch(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    @expectedFailureAll(archs=["i386"])
    @skipIfRemote
    def test(self):
        """
        Test lldb's parsing of the <architecture> tag in the target.xml register
        description packet.
        """
        self.server.responder = MyResponder()
        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        target = self.dbg.CreateTarget("")
        self.assertEqual("", target.GetTriple())
        process = self.connect(target)
        if self.TraceOn():
            interp.HandleCommand("target list", result)
            print(result.GetOutput())
        self.assertTrue(target.GetTriple().startswith("x86_64-unknown-unknown"))

    @skipIfXmlSupportMissing
    @skipIfRemote
    @skipIfLLVMTargetMissing("X86")
    def test_register_augmentation(self):
        """
        Test that we correctly associate the register info with the eh_frame
        register numbers.
        """

        target = self.createTarget("basic_eh_frame.yaml")
        self.server.responder = MyResponder()

        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )
        self.filecheck("image show-unwind -n foo", __file__, "--check-prefix=UNWIND")


# UNWIND: eh_frame UnwindPlan:
# UNWIND: row[0]:    0: CFA=rsp+128 => rip=[CFA-8]



"""
Test that we correctly handle namespace
expression evaluation through namespace
aliases.
"""

import lldb

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestInlineNamespace(TestBase):
    @skipIf(compiler="clang", compiler_version=["<", "16.0"])
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "return A::B::C::a", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr("A::C::a", result_type="int", result_value="-1")
        self.expect_expr("A::D::a", result_type="int", result_value="-1")

        self.expect_expr("A::C::func()", result_type="int", result_value="0")
        self.expect_expr("A::D::func()", result_type="int", result_value="0")

        self.expect_expr("E::C::a", result_type="int", result_value="-1")
        self.expect_expr("E::D::a", result_type="int", result_value="-1")
        self.expect_expr("F::a", result_type="int", result_value="-1")
        self.expect_expr("G::a", result_type="int", result_value="-1")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestOperatorOverload(TestBase):
    def test_overload(self):
        self.build()
        (target, process, thread, main_breakpoint) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("b.cpp")
        )
        frame = thread.GetSelectedFrame()
        value = frame.EvaluateExpression("x == nil")
        self.assertFalse(value.GetError().Success())
        self.assertIn(
            "comparison between NULL and non-pointer ('Tinky' and NULL)",
            str(value.GetError()),
        )
        self.assertIn(
            "invalid operands to binary expression ('Tinky' and", str(value.GetError())
        )



"""
Test how lldb reacts to ambiguous commands
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AmbiguousCommandTestCase(TestBase):
    @no_debug_info_test
    def test_ambiguous_command_with_alias(self):
        command_interpreter = self.dbg.GetCommandInterpreter()
        self.assertTrue(command_interpreter, VALID_COMMAND_INTERPRETER)
        result = lldb.SBCommandReturnObject()

        command_interpreter.HandleCommand(
            "command alias corefile target create -c %0", result
        )
        self.assertTrue(result.Succeeded())

        command_interpreter.ResolveCommand("co", result)
        self.assertFalse(result.Succeeded())
        self.assertEqual(
            result.GetError(),
            "error: ambiguous command 'co'. Possible matches:\n\tcommand\n\tcontinue\n\tcorefile\n",
        )

        command_interpreter.HandleCommand("command unalias continue", result)
        self.assertTrue(result.Succeeded())

        command_interpreter.ResolveCommand("co", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual(result.GetOutput(), "target create -c %0")



"""
Test using LLDB data formatters with frozen objects coming from the expression parser.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprFormattersTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.cpp.
        self.line = line_number("main.cpp", "// Stop here")

    @skipIfTargetAndroid()  # skipping to avoid crashing the test runner
    @expectedFailureAndroid("llvm.org/pr24691")  # we hit an assertion in clang
    def test(self):
        """Test expr + formatters for good interoperability."""
        self.build()

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synthetic clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        """Test expr + formatters for good interoperability."""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)
        self.runCmd("command script import formatters.py")
        self.runCmd("command script import foosynth.py")

        if self.TraceOn():
            self.runCmd("frame variable foo1 --show-types")
            self.runCmd("frame variable foo1.b --show-types")
            self.runCmd("frame variable foo1.b.b_ref --show-types")

        self.filecheck(
            "expression --show-types -- *(new_foo(47))",
            __file__,
            "-check-prefix=EXPR-TYPES-NEW-FOO",
        )
        # EXPR-TYPES-NEW-FOO: (foo) ${{.*}} = {
        # EXPR-TYPES-NEW-FOO-NEXT:   (int) a = 47
        # EXPR-TYPES-NEW-FOO-NEXT:   (int *) a_ptr = 0x
        # EXPR-TYPES-NEW-FOO-NEXT:   (bar) b = {
        # EXPR-TYPES-NEW-FOO-NEXT:     (int) i = 94
        # EXPR-TYPES-NEW-FOO-NEXT:     (int *) i_ptr = 0x
        # EXPR-TYPES-NEW-FOO-NEXT:     (baz) b = {
        # EXPR-TYPES-NEW-FOO-NEXT:       (int) h = 97
        # EXPR-TYPES-NEW-FOO-NEXT:       (int) k = 99
        # EXPR-TYPES-NEW-FOO-NEXT:     }
        # EXPR-TYPES-NEW-FOO-NEXT:     (baz &) b_ref = 0x
        # EXPR-TYPES-NEW-FOO-NEXT:   }
        # EXPR-TYPES-NEW-FOO-NEXT: }

        self.runCmd("type summary add -F formatters.foo_SummaryProvider3 foo")
        self.filecheck("expression foo1", __file__, "-check-prefix=EXPR-FOO1opts")
        # EXPR-FOO1opts: (foo) $
        # EXPR-FOO1opts-SAME: a = 12
        # EXPR-FOO1opts-SAME: a_ptr = {{[0-9]+}} -> 13
        # EXPR-FOO1opts-SAME: i = 24
        # EXPR-FOO1opts-SAME: i_ptr = {{[0-9]+}} -> 25
        # EXPR-FOO1opts-SAME: b_ref = {{[0-9]+}}
        # EXPR-FOO1opts-SAME: h = 27
        # EXPR-FOO1opts-SAME: k = 29
        # EXPR-FOO1opts-SAME: WITH_OPTS

        self.runCmd("type summary delete foo")

        self.runCmd("type summary add -F formatters.foo_SummaryProvider foo")

        self.expect("expression new_int(12)", substrs=["(int *) $", " = 0x"])

        self.runCmd('type summary add -s "${var%pointer} -> ${*var%decimal}" "int *"')

        self.expect("expression new_int(12)", substrs=["(int *) $", "= 0x", " -> 12"])

        self.expect("expression foo1.a_ptr", substrs=["(int *) $", "= 0x", " -> 13"])

        self.filecheck("expression foo1", __file__, "-check-prefix=EXPR-FOO1")
        # EXPR-FOO1: (foo) $
        # EXPR-FOO1-SAME: a = 12
        # EXPR-FOO1-SAME: a_ptr = {{[0-9]+}} -> 13
        # EXPR-FOO1-SAME: i = 24
        # EXPR-FOO1-SAME: i_ptr = {{[0-9]+}} -> 25
        # EXPR-FOO1-SAME: b_ref = {{[0-9]+}}
        # EXPR-FOO1-SAME: h = 27
        # EXPR-FOO1-SAME: k = 29

        self.filecheck(
            "expression --ptr-depth=1 -- new_foo(47)",
            __file__,
            "-check-prefix=EXPR-PTR-DEPTH1",
        )
        # EXPR-PTR-DEPTH1: (foo *) $
        # EXPR-PTR-DEPTH1-SAME: a = 47
        # EXPR-PTR-DEPTH1-SAME: a_ptr = {{[0-9]+}} -> 48
        # EXPR-PTR-DEPTH1-SAME: i = 94
        # EXPR-PTR-DEPTH1-SAME: i_ptr = {{[0-9]+}} -> 95

        self.filecheck("expression foo2", __file__, "-check-prefix=EXPR-FOO2")
        # EXPR-FOO2: (foo) $
        # EXPR-FOO2-SAME: a = 121
        # EXPR-FOO2-SAME: a_ptr = {{[0-9]+}} -> 122
        # EXPR-FOO2-SAME: i = 242
        # EXPR-FOO2-SAME: i_ptr = {{[0-9]+}} -> 243
        # EXPR-FOO2-SAME: h = 245
        # EXPR-FOO2-SAME: k = 247

        object_name = self.res.GetOutput()
        object_name = object_name[7:]
        object_name = object_name[0 : object_name.find(" =")]

        self.filecheck("frame variable foo2", __file__, "-check-prefix=VAR-FOO2")
        # VAR-FOO2: (foo) foo2
        # VAR-FOO2-SAME: a = 121
        # VAR-FOO2-SAME: a_ptr = {{[0-9]+}} -> 122
        # VAR-FOO2-SAME: i = 242
        # VAR-FOO2-SAME: i_ptr = {{[0-9]+}} -> 243
        # VAR-FOO2-SAME: h = 245
        # VAR-FOO2-SAME: k = 247

        # The object is the same as foo2, so use the EXPR-FOO2 checks.
        self.filecheck(
            "expression $" + object_name, __file__, "-check-prefix=EXPR-FOO2"
        )

        self.runCmd("type summary delete foo")
        self.runCmd(
            "type synthetic add --python-class foosynth.FooSyntheticProvider foo"
        )

        self.expect(
            "expression --show-types -- $" + object_name,
            substrs=["(foo) $", " = {", "(int) *i_ptr = 243"],
        )

        self.runCmd("n")
        self.runCmd("n")

        self.runCmd("type synthetic delete foo")
        self.runCmd("type summary add -F formatters.foo_SummaryProvider foo")

        self.expect(
            "expression foo2",
            substrs=[
                "(foo) $",
                "a = 7777",
                "a_ptr = ",
                " -> 122",
                "i = 242",
                "i_ptr = ",
                " -> 8888",
            ],
        )

        self.expect("expression $" + object_name + ".a", substrs=["7777"])

        self.expect("expression *$" + object_name + ".b.i_ptr", substrs=["8888"])

        self.expect(
            "expression $" + object_name,
            substrs=[
                "(foo) $",
                "a = 121",
                "a_ptr = ",
                " -> 122",
                "i = 242",
                "i_ptr = ",
                " -> 8888",
                "h = 245",
                "k = 247",
            ],
        )

        self.runCmd("type summary delete foo")
        self.runCmd(
            "type synthetic add --python-class foosynth.FooSyntheticProvider foo"
        )

        self.expect(
            "expression --show-types -- $" + object_name,
            substrs=["(foo) $", " = {", "(int) *i_ptr = 8888"],
        )

        self.runCmd("n")

        self.runCmd("type synthetic delete foo")
        self.runCmd("type summary add -F formatters.foo_SummaryProvider foo")

        self.expect(
            "expression $" + object_name,
            substrs=[
                "(foo) $",
                "a = 121",
                "a_ptr = ",
                " -> 122",
                "i = 242",
                "i_ptr = ",
                " -> 8888",
                "k = 247",
            ],
        )

        process = self.dbg.GetSelectedTarget().GetProcess()
        thread = process.GetThreadAtIndex(0)
        frame = thread.GetSelectedFrame()

        frozen = frame.EvaluateExpression("$" + object_name + ".a_ptr")

        a_data = frozen.GetPointeeData()

        error = lldb.SBError()
        self.assertEqual(a_data.GetUnsignedInt32(error, 0), 122, "*a_ptr = 122")

        ret = line_number("main.cpp", "Done initializing")
        self.runCmd("thread until " + str(ret))

        self.expect("frame variable numbers", substrs=["1", "2", "3", "4", "5"])

        self.expect("expression numbers", substrs=["1", "2", "3", "4", "5"])

        frozen = frame.EvaluateExpression("&numbers")

        a_data = frozen.GetPointeeData(0, 1)

        self.assertEqual(a_data.GetUnsignedInt32(error, 0), 1, "numbers[0] == 1")
        self.assertEqual(a_data.GetUnsignedInt32(error, 4), 2, "numbers[1] == 2")
        self.assertEqual(a_data.GetUnsignedInt32(error, 8), 3, "numbers[2] == 3")
        self.assertEqual(a_data.GetUnsignedInt32(error, 12), 4, "numbers[3] == 4")
        self.assertEqual(a_data.GetUnsignedInt32(error, 16), 5, "numbers[4] == 5")

        frozen = frame.EvaluateExpression("numbers")

        a_data = frozen.GetData()

        self.assertEqual(a_data.GetUnsignedInt32(error, 0), 1, "numbers[0] == 1")
        self.assertEqual(a_data.GetUnsignedInt32(error, 4), 2, "numbers[1] == 2")
        self.assertEqual(a_data.GetUnsignedInt32(error, 8), 3, "numbers[2] == 3")
        self.assertEqual(a_data.GetUnsignedInt32(error, 12), 4, "numbers[3] == 4")
        self.assertEqual(a_data.GetUnsignedInt32(error, 16), 5, "numbers[4] == 5")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestPlatformAttach(GDBRemoteTestBase):
    @skipIfRemote
    @skipIfWindows
    def test_attach(self):
        """Test attaching by name"""

        class MyPlatformResponder(MockGDBServerResponder):
            def __init__(self, port):
                MockGDBServerResponder.__init__(self)
                self.port = port

            def qLaunchGDBServer(self, _):
                return "pid:1337;port:{};".format(self.port)

            def qfProcessInfo(self, packet):
                return "pid:95117;name:666f6f;"

        class MyGDBResponder(MockGDBServerResponder):
            def __init__(self):
                MockGDBServerResponder.__init__(self)

            def vAttach(self, _):
                return "OK"

        self.server.responder = MyGDBResponder()
        port = self.server._socket._server_socket.getsockname()[1]

        platform_socket = TCPServerSocket()
        platform_server = MockGDBServer(platform_socket)
        platform_server.responder = MyPlatformResponder(port)
        platform_server.start()

        error = lldb.SBError()
        platform = lldb.SBPlatform("remote-linux")
        self.dbg.SetSelectedPlatform(platform)

        error = platform.ConnectRemote(
            lldb.SBPlatformConnectOptions(platform_server.get_connect_url())
        )
        self.assertSuccess(error)
        self.assertTrue(platform.IsConnected())

        attach_info = lldb.SBAttachInfo()
        attach_info.SetExecutable("foo")
        attach_info.SetProcessPluginName("wasm")

        target = lldb.SBTarget()
        process = platform.Attach(attach_info, self.dbg, target, error)
        self.assertSuccess(error)
        self.assertEqual(process.GetProcessID(), 95117)
        self.assertEqual(process.GetPluginName(), "wasm")

        platform.DisconnectRemote()



import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestCase(TestBase):
    def test_functions_having_dlang_mangling_prefix(self):
        """
        Ensure C functions with a '_D' prefix alone are not mistakenly treated
        as a Dlang mangled name. A proper Dlang mangling will have digits
        immediately following the '_D' prefix.
        """
        self.build()
        _, _, thread, _ = lldbutil.run_to_name_breakpoint(self, "_Dfunction")
        frame = thread.frame[0]

        symbol = frame.symbol
        # On Windows the function does not have an associated symbol.
        if symbol.IsValid():
            self.assertFalse(symbol.mangled)
            self.assertEqual(symbol.GetDisplayName(), "_Dfunction")

        function = frame.function
        self.assertFalse(function.mangled)
        self.assertEqual(function.GetDisplayName(), "_Dfunction")



from lldbsuite.test import lldbinline
from lldbsuite.test import decorators

lldbinline.MakeInlineTest(
    __file__,
    globals(),
    [lldbinline.expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr43707")],
)



"""
Make sure running internal expressions doesn't
influence the result variable numbering.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestExpressionResultNumbering(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_sample_rename_this(self):
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.do_numbering_test()

    def do_numbering_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        bkpt = target.BreakpointCreateBySourceRegex(
            "Add conditions to this breakpoint", self.main_source_file
        )
        self.assertEqual(bkpt.GetNumLocations(), 1, "Set the breakpoint")

        bkpt.SetCondition("call_me(value) < 6")

        # Get the number of the last expression:
        result = thread.frames[0].EvaluateExpression("call_me(200)")
        self.assertSuccess(result.GetError(), "Our expression succeeded")
        name = result.GetName()
        ordinal = int(name[1:])

        process.Continue()

        # The condition evaluation had to run a 4 expressions, but we haven't
        # run any user expressions.
        result = thread.frames[0].EvaluateExpression("call_me(200)")
        self.assertSuccess(
            result.GetError(), "Our expression succeeded the second time"
        )
        after_name = result.GetName()
        after_ordinal = int(after_name[1:])
        self.assertEqual(ordinal + 1, after_ordinal)



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import xegpu
from mlir.dialects.transform import structured, AnyValueType


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f()
        print(module)
    return f


@run
def getLoadOp():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [0])
        load_handle = xegpu.get_load_op(operand)
        transform.YieldOp()
    # CHECK-LABEL: TEST: getLoadOp
    # CHECK: transform.xegpu.get_load_op %


@run
def setAnchorLayout():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.load_nd"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_anchor_layout(
            sequence.bodyTarget,
            sg_layout=[6, 4],
            sg_data=[32, 16],
            inst_data=[8, 16],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setAnchorLayout
    # CHECK: transform.xegpu.set_anchor_layout %
    # CHECK-NOT: index = 0
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # CHECK: inst_data = [8, 16]


@run
def setAnchorLayoutDPAS():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_anchor_layout(
            sequence.bodyTarget,
            index=1,
            sg_layout=[6, 4],
            sg_data=[32, 16],
            inst_data=[8, 16],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setAnchorLayoutDPAS
    # CHECK: transform.xegpu.set_anchor_layout %
    # CHECK: index = 1
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # CHECK: inst_data = [8, 16]


@run
def setAnchorLayoutOrder():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.load_nd"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_anchor_layout(
            sequence.bodyTarget,
            sg_layout=[6, 4],
            sg_data=[32, 16],
            inst_data=[8, 16],
            order=[1, 0],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setAnchorLayoutOrder
    # CHECK: transform.xegpu.set_anchor_layout %
    # CHECK-NOT: index = 0
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # CHECK: inst_data = [8, 16]
    # CHECK: order = [1, 0]


@run
def setAnchorLayoutSlice():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.load"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_anchor_layout(
            sequence.bodyTarget,
            sg_layout=[6, 4],
            sg_data=[32, 16],
            inst_data=[8, 16],
            slice_dims=[0],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setAnchorLayoutSlice
    # CHECK: transform.xegpu.set_anchor_layout %
    # CHECK-NOT: index = 0
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # CHECK: inst_data = [8, 16]
    # CHECK: slice_dims = [0]


@run
def setGPULaunchThreadsOp():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("gpu.launch"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_gpu_launch_threads(sequence.bodyTarget, threads=[8, 4, 1])
        transform.YieldOp()
    # CHECK-LABEL: TEST: setGPULaunchThreadsOp
    # CHECK: transform.xegpu.set_gpu_launch_threads
    # CHECK: threads = [8, 4, 1]


@run
def insertPrefetch():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.load_nd"),
    )
    with InsertionPoint(sequence.body):
        xegpu.insert_prefetch(sequence.bodyTarget)
        transform.YieldOp()
    # CHECK-LABEL: TEST: insertPrefetch
    # CHECK: transform.xegpu.insert_prefetch


@run
def insertPrefetchNbPrefetch():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.load_nd"),
    )
    with InsertionPoint(sequence.body):
        xegpu.insert_prefetch(sequence.bodyTarget, nb_prefetch=2)
        transform.YieldOp()
    # CHECK-LABEL: TEST: insertPrefetchNbPrefetch
    # CHECK: transform.xegpu.insert_prefetch
    # CHECK-SAME: nb_prefetch = 2


@run
def insertPrefetchNbPrefetchParam():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.load_nd"),
    )
    with InsertionPoint(sequence.body):
        int32_t = IntegerType.get_signless(32)
        param_int32_t = transform.ParamType.get(int32_t)
        nb_param = transform.ParamConstantOp(
            param_int32_t,
            IntegerAttr.get(int32_t, 2),
        )
        xegpu.insert_prefetch(sequence.bodyTarget, nb_prefetch=nb_param)
        transform.YieldOp()
    # CHECK-LABEL: TEST: insertPrefetchNbPrefetchParam
    # CHECK: %[[PARAM_OP:.*]] = transform.param.constant 2
    # CHECK: transform.xegpu.insert_prefetch
    # CHECK-SAME: nb_prefetch = %[[PARAM_OP]]


@run
def ConvertLayoutMinimal():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [0])
        xegpu.convert_layout(
            operand,
            input_sg_layout=[6, 4],
            input_sg_data=[32, 16],
            target_sg_layout=[6, 4],
            target_sg_data=[8, 16],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: ConvertLayoutMinimal
    # CHECK: transform.xegpu.convert_layout %
    # CHECK: input_sg_layout = [6, 4]
    # CHECK: input_sg_data = [32, 16]
    # CHECK: target_sg_layout = [6, 4]
    # CHECK: target_sg_data = [8, 16]


@run
def ConvertLayout():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [1])
        xegpu.convert_layout(
            operand,
            input_sg_layout=[6, 4],
            input_sg_data=[32, 32],
            input_inst_data=[32, 16],
            input_order=[1, 0],
            target_sg_layout=[6, 4],
            target_sg_data=[32, 32],
            target_inst_data=[8, 16],
            target_order=[0, 1],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: ConvertLayout
    # CHECK: transform.xegpu.convert_layout %
    # CHECK: input_sg_layout = [6, 4]
    # CHECK: input_sg_data = [32, 32]
    # CHECK: input_inst_data = [32, 16]
    # CHECK: input_order = [1, 0]
    # CHECK: target_sg_layout = [6, 4]
    # CHECK: target_sg_data = [32, 32]
    # CHECK: target_inst_data = [8, 16]
    # CHECK: target_order = [0, 1]



"""
Test the robustness of lldb expression parser.
"""


import lldb
from lldbsuite.test.lldbtest import *


class Radar8638051TestCase(TestBase):
    def test_expr_commands(self):
        """The following expression commands should not crash."""
        self.build()

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        self.runCmd("breakpoint set -n c")

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect_expr("val", result_type="int", result_value="1")
        self.expect_expr("*(&val)", result_type="int", result_value="1")

        # rdar://problem/8638051
        # lldb expression command: Could this crash be avoided
        self.expect("expression &val", startstr="(int *) $2 = ")
        # (int *) $2 = 0x....



"""
Test that step over will let other threads run when necessary
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StepOverDoesntDeadlockTestCase(TestBase):
    def test_step_over(self):
        """Test that when step over steps over a function it lets other threads run."""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self,
            "without running the first thread at least somewhat",
            lldb.SBFileSpec("locking.cpp"),
        )
        # This is just testing that the step over actually completes.
        # If the test fails this step never return, so failure is really
        # signaled by the test timing out.

        thread.StepOver()
        state = process.GetState()
        self.assertState(state, lldb.eStateStopped)



from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentTwoBreakpointsOneDelaySignal(ConcurrentEventsBase):
    # Atomic sequences are not supported yet for MIPS in LLDB.
    @skipIf(triple="^mips")
    @expectedFlakeyNetBSD
    @expectedFailureAll(
        archs=["aarch64"], oslist=["freebsd"], bugnumber="llvm.org/pr49433"
    )
    def test(self):
        """Test two threads that trigger a breakpoint and one (1 second delay) signal thread."""
        self.build()
        self.do_thread_actions(num_breakpoint_threads=2, num_delay_signal_threads=1)



"""
Test lldb's support for the AArch64 Permission Overlay extension (POE), which
is used to implement Linux's memory protection keys feature.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64LinuxPOE(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    EXPECTED_POR = "por = 0x0000000001234567"
    EXPECTED_POR_FIELDS = (
        "         = {\n"
        "             Perm15 = No Access\n"
        "             Perm14 = No Access\n"
        "             Perm13 = No Access\n"
        "             Perm12 = No Access\n"
        "             Perm11 = No Access\n"
        "             Perm10 = No Access\n"
        "             Perm9 = No Access\n"
        "             Perm8 = No Access\n"
        "             Perm7 = No Access\n"
        "             Perm6 = Read\n"
        "             Perm5 = Execute\n"
        "             Perm4 = Read, Execute\n"
        "             Perm3 = Write\n"
        "             Perm2 = Write, Read\n"
        "             Perm1 = Write, Execute\n"
        "             Perm0 = Read, Write, Execute\n"
        "           }"
    )

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_poe_live(self):
        if not self.isAArch64POE():
            self.skipTest("POE must be present.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.expect(
            "register read --all",
            substrs=[
                "Permission Overlay Registers",
                f"{self.EXPECTED_POR}",
            ],
        )

        if self.hasXMLSupport():
            self.expect(
                "register read por",
                substrs=[f"     {self.EXPECTED_POR}\n" + self.EXPECTED_POR_FIELDS],
            )

        # POR should be restored after expression evaluation.
        self.expect("expression expr_function()", substrs=["$0 = 1"])
        self.expect("register read por", substrs=[self.EXPECTED_POR])

        # Not passing this to the application allows us to fix the permissions
        # using lldb, then continue to a normal exit.
        self.runCmd("process handle SIGSEGV --pass false")

        self.expect(
            "continue",
            substrs=[
                "stop reason = signal SIGSEGV: failed protection key checks (fault address="
            ],
        )

        # This fault should have happened due to the write, not the read.
        self.assertEqual(
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
            .GetFunctionName(),
            "cause_write_fault",
        )

        # Allow writes so we can continue. This value has permission 6 changed
        # from read only (1) to write (4).
        self.runCmd("register write por 0x4234567")

        self.expect("continue", substrs=["exited with status = 0"])

    @skipIfLLVMTargetMissing("AArch64")
    def test_poe_core(self):
        # Core was generated by running the test program on a system with POE
        # and coredump_filter set to 0.
        self.runCmd("target create --core corefile")

        self.expect("process status", substrs=["SIGSEGV: failed protection key checks"])

        self.expect(
            "register read --all",
            substrs=[
                "Permission Overlay Registers",
                f"{self.EXPECTED_POR}",
            ],
        )

        if self.hasXMLSupport():
            self.expect(
                "register read por",
                substrs=[f"     {self.EXPECTED_POR}\n" + self.EXPECTED_POR_FIELDS],
            )



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import sys
import lldb


def fuzz_obj(obj):
    obj.GetFileSpec()
    obj.GetPlatformFileSpec()
    obj.SetPlatformFileSpec(lldb.SBFileSpec())
    obj.GetUUIDString()
    obj.ResolveFileAddress(sys.maxsize)
    obj.ResolveSymbolContextForAddress(lldb.SBAddress(), 0)
    obj.GetDescription(lldb.SBStream())
    obj.GetNumSymbols()
    obj.GetSymbolAtIndex(sys.maxsize)
    sc_list = obj.FindFunctions("my_func")
    sc_list = obj.FindFunctions("my_func", lldb.eFunctionNameTypeAny)
    obj.FindGlobalVariables(lldb.SBTarget(), "my_global_var", 1)
    for section in obj.section_iter():
        s = str(section)
    for symbol in obj.symbol_in_section_iter(lldb.SBSection()):
        s = str(symbol)
    for symbol in obj:
        s = str(symbol)
    obj.GetAddressByteSize()
    obj.GetByteOrder()
    obj.GetTriple()



"""Test that types defined in shared libraries work correctly."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestRealDefinition(TestBase):
    def test_frame_var_after_stop_at_implementation(self):
        """Test that we can find the implementation for an objective C type"""
        if self.getArchitecture() == "i386":
            self.skipTest("requires modern objc runtime")
        self.build()
        self.shlib_names = ["libTestExt.dylib", "libTest.dylib"]
        self.common_setup()

        line = line_number("TestExt/TestExt.m", "// break here")
        lldbutil.run_break_set_by_file_and_line(
            self, "TestExt.m", line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # This should display correctly.
        self.expect(
            "expr 42", "A simple expression should execute correctly", substrs=["42"]
        )

    def common_setup(self):
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.registerSharedLibrariesWithTarget(target, self.shlib_names)

        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)



# RUN: %PYTHON %s 2>&1 | FileCheck %s

import gc
from mlir.ir import *
from mlir.passmanager import *
from mlir.dialects.builtin import ModuleOp
from mlir.dialects import arith
from mlir.rewrite import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    return f


# CHECK-LABEL: TEST: testRewritePattern
@run
def testRewritePattern():
    def to_muli(op, rewriter):
        with rewriter.ip:
            assert isinstance(op, arith.AddIOp)
            new_op = arith.muli(op.lhs, op.rhs, loc=op.location)
        rewriter.replace_op(op, new_op.owner)

    def constant_1_to_2(op, rewriter):
        c = op.value.value
        if c != 1:
            return True  # failed to match
        with rewriter.ip:
            new_op = arith.constant(op.type, 2, loc=op.location)
        rewriter.replace_op(op, [new_op])

    with Context():
        patterns = RewritePatternSet()
        patterns.add(arith.AddIOp, to_muli)
        patterns.add("arith.constant", constant_1_to_2)
        frozen = patterns.freeze()

        module = ModuleOp.parse(
            r"""
            module {
              func.func @add(%a: i64, %b: i64) -> i64 {
                %sum = arith.addi %a, %b : i64
                return %sum : i64
              }
            }
            """
        )

        apply_patterns_and_fold_greedily(module, frozen)
        # CHECK: %0 = arith.muli %arg0, %arg1 : i64
        # CHECK: return %0 : i64
        print(module)

        module = ModuleOp.parse(
            r"""
            module {
              func.func @const() -> (i64, i64) {
                %0 = arith.constant 1 : i64
                %1 = arith.constant 3 : i64
                return %0, %1 : i64, i64
              }
            }
            """
        )

        apply_patterns_and_fold_greedily(module, frozen)
        # CHECK: %c2_i64 = arith.constant 2 : i64
        # CHECK: %c3_i64 = arith.constant 3 : i64
        # CHECK: return %c2_i64, %c3_i64 : i64, i64
        print(module)

        module = ModuleOp.parse(
            r"""
            module {
              func.func @add(%a: i64, %b: i64) -> i64 {
                %sum = arith.addi %a, %b : i64
                return %sum : i64
              }
            }
            """
        )

        walk_and_apply_patterns(module, frozen)
        # CHECK: %0 = arith.muli %arg0, %arg1 : i64
        # CHECK: return %0 : i64
        print(module)


# CHECK-LABEL: TEST: testGreedyRewriteConfigCreation
@run
def testGreedyRewriteConfigCreation():
    # Test basic config creation and destruction
    config = GreedyRewriteConfig()
    # CHECK: Config created successfully
    print("Config created successfully")


# CHECK-LABEL: TEST: testGreedyRewriteConfigGetters
@run
def testGreedyRewriteConfigGetters():
    config = GreedyRewriteConfig()

    # Set some values
    config.max_iterations = 5
    config.max_num_rewrites = 50
    config.use_top_down_traversal = True
    config.enable_folding = False
    config.strictness = GreedyRewriteStrictness.EXISTING_AND_NEW_OPS
    config.region_simplification_level = GreedySimplifyRegionLevel.AGGRESSIVE
    config.enable_constant_cse = True

    # Test all getter methods and print results
    # CHECK: max_iterations: 5
    max_iterations = config.max_iterations
    print(f"max_iterations: {max_iterations}")
    # CHECK: max_rewrites: 50
    max_rewrites = config.max_num_rewrites
    print(f"max_rewrites: {max_rewrites}")
    # CHECK: use_top_down: True
    use_top_down = config.use_top_down_traversal
    print(f"use_top_down: {use_top_down}")
    # CHECK: folding_enabled: False
    folding_enabled = config.enable_folding
    print(f"folding_enabled: {folding_enabled}")
    # CHECK: strictness: GreedyRewriteStrictness.EXISTING_AND_NEW_OPS
    strictness = config.strictness
    print(f"strictness: {strictness}")
    # CHECK: region_level: GreedySimplifyRegionLevel.AGGRESSIVE
    region_level = config.region_simplification_level
    print(f"region_level: {region_level}")
    # CHECK: cse_enabled: True
    cse_enabled = config.enable_constant_cse
    print(f"cse_enabled: {cse_enabled}")


# CHECK-LABEL: TEST: testGreedyRewriteStrictnessEnum
@run
def testGreedyRewriteStrictnessEnum():
    config = GreedyRewriteConfig()

    # Test ANY_OP
    # CHECK: strictness ANY_OP: GreedyRewriteStrictness.ANY_OP
    config.strictness = GreedyRewriteStrictness.ANY_OP
    strictness = config.strictness
    print(f"strictness ANY_OP: {strictness}")

    # Test EXISTING_AND_NEW_OPS
    # CHECK: strictness EXISTING_AND_NEW_OPS: GreedyRewriteStrictness.EXISTING_AND_NEW_OPS
    config.strictness = GreedyRewriteStrictness.EXISTING_AND_NEW_OPS
    strictness = config.strictness
    print(f"strictness EXISTING_AND_NEW_OPS: {strictness}")

    # Test EXISTING_OPS
    # CHECK: strictness EXISTING_OPS: GreedyRewriteStrictness.EXISTING_OPS
    config.strictness = GreedyRewriteStrictness.EXISTING_OPS
    strictness = config.strictness
    print(f"strictness EXISTING_OPS: {strictness}")


# CHECK-LABEL: TEST: testGreedySimplifyRegionLevelEnum
@run
def testGreedySimplifyRegionLevelEnum():
    config = GreedyRewriteConfig()

    # Test DISABLED
    # CHECK: region_level DISABLED: GreedySimplifyRegionLevel.DISABLED
    config.region_simplification_level = GreedySimplifyRegionLevel.DISABLED
    level = config.region_simplification_level
    print(f"region_level DISABLED: {level}")

    # Test NORMAL
    # CHECK: region_level NORMAL: GreedySimplifyRegionLevel.NORMAL
    config.region_simplification_level = GreedySimplifyRegionLevel.NORMAL
    level = config.region_simplification_level
    print(f"region_level NORMAL: {level}")

    # Test AGGRESSIVE
    # CHECK: region_level AGGRESSIVE: GreedySimplifyRegionLevel.AGGRESSIVE
    config.region_simplification_level = GreedySimplifyRegionLevel.AGGRESSIVE
    level = config.region_simplification_level
    print(f"region_level AGGRESSIVE: {level}")


# CHECK-LABEL: TEST: testRewriteWithGreedyRewriteConfig
@run
def testRewriteWithGreedyRewriteConfig():
    def constant_1_to_2(op, rewriter):
        c = op.value.value
        if c != 1:
            return True  # failed to match
        with rewriter.ip:
            new_op = arith.constant(op.type, 2, loc=op.location)
        rewriter.replace_op(op, [new_op])

    with Context():
        patterns = RewritePatternSet()
        patterns.add(arith.ConstantOp, constant_1_to_2)
        frozen = patterns.freeze()

        module = ModuleOp.parse(
            r"""
            module {
              func.func @const() -> (i64, i64) {
                %0 = arith.constant 1 : i64
                %1 = arith.constant 1 : i64
                return %0, %1 : i64, i64
              }
            }
            """
        )

        config = GreedyRewriteConfig()
        config.enable_constant_cse = False
        apply_patterns_and_fold_greedily(module, frozen, config)
        # CHECK: %c2_i64 = arith.constant 2 : i64
        # CHECK: %c2_i64_0 = arith.constant 2 : i64
        # CHECK: return %c2_i64, %c2_i64_0 : i64, i64
        print(module)

        config = GreedyRewriteConfig()
        config.enable_constant_cse = True
        apply_patterns_and_fold_greedily(module, frozen, config)
        # CHECK: %c2_i64 = arith.constant 2 : i64
        # CHECK: return %c2_i64, %c2_i64 : i64
        print(module)


@run
def testConversionPattern():
    from mlir.dialects import smt

    def convert_int(t):
        if isinstance(t, IntegerType):
            return smt.IntType.get()

    converter = TypeConverter()
    converter.add_conversion(convert_int)

    def convert_constant(op, adaptor, type_converter, rewriter):
        assert isinstance(op, arith.ConstantOp)
        assert isinstance(adaptor, arith.ConstantOpAdaptor)
        with rewriter.ip:
            new_op = smt.IntConstantOp(op.value, loc=op.location)
        rewriter.replace_op(op, new_op)

    def convert_addi(op, adaptor, type_converter, rewriter):
        assert isinstance(op, arith.AddIOp)
        assert isinstance(adaptor, arith.AddIOpAdaptor)
        with rewriter.ip:
            new_op = smt.IntAddOp([adaptor.lhs, adaptor.rhs], loc=op.location)
        rewriter.replace_op(op, new_op)

    def convert_muli(op, adaptor, type_converter, rewriter):
        assert isinstance(op, arith.MulIOp)
        assert isinstance(adaptor, arith.MulIOpAdaptor)
        with rewriter.ip:
            new_op = smt.IntMulOp([adaptor.lhs, adaptor.rhs], loc=op.location)
        rewriter.replace_op(op, new_op)

    with Context():
        patterns = RewritePatternSet()
        patterns.add_conversion(arith.ConstantOp, convert_constant, converter)
        patterns.add_conversion(arith.AddIOp, convert_addi, converter)
        patterns.add_conversion(arith.MulIOp, convert_muli, converter)

        module = ModuleOp.parse(
            r"""
            module {
                func.func @f(%0: i64) -> i64 {
                    %1 = arith.constant 3 : i64
                    %2 = arith.addi %0, %1 : i64
                    %3 = arith.muli %2, %1 : i64
                    return %3 : i64
                }
            }
            """
        )

        target = ConversionTarget()
        target.add_legal_dialect(smt._Dialect)
        target.add_illegal_op(arith.ConstantOp, arith.AddIOp, arith.MulIOp)

        frozen = patterns.freeze()
        config = ConversionConfig()
        config.build_materializations = False

        apply_partial_conversion(module, target, frozen, config)
        assert module.operation.verify()

        # CHECK: func.func @f(%arg0: i64) -> i64 {
        # CHECK:     %0 = builtin.unrealized_conversion_cast %arg0 : i64 to !smt.int
        # CHECK:     %c3 = smt.int.constant 3
        # CHECK:     %1 = smt.int.add %0, %c3
        # CHECK:     %2 = smt.int.mul %1, %c3
        # CHECK:     %3 = builtin.unrealized_conversion_cast %2 : !smt.int to i64
        # CHECK:     return %3 : i64
        # CHECK: }
        print(module)

        module = ModuleOp.parse(
            r"""
            module {
                func.func @f(%0: i64) -> i64 {
                    %1 = arith.constant 3 : i64
                    %2 = arith.addi %0, %1 : i64
                    %3 = arith.muli %2, %1 : i64
                    return %3 : i64
                }
            }
            """
        )
        try:
            apply_partial_conversion(module, target, frozen)
        except MLIRError as e:
            # CHECK: caught exception: partial conversion failed
            # CHECK: failed to legalize unresolved materialization
            print("caught exception:", e)

        t1 = converter.convert_type(IntegerType.get_signless(64))
        # CHECK: IntType
        print(type(t1))
        # CHECK: !smt.int
        print(str(t1))
        t2 = converter.convert_type(F32Type.get())
        # CHECK: None
        print(t2)



import re
import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from os.path import exists


def get_os_version():
    try:
        os_version_str = subprocess.check_output(["sysctl", "kern.osversion"]).decode(
            "utf-8"
        )
    except subprocess.CalledProcessError:
        return None
    m = re.match(r"kern\.osversion: (\w+)", os_version_str)
    if m:
        return m.group(1)
    return None


def rosetta_debugserver_installed():
    return exists("/Library/Apple/usr/libexec/oah/debugserver")


def has_rosetta_shared_cache(os_version):
    if not os_version:
        return False
    macos_device_support = os.path.join(
        os.path.expanduser("~"), "Library", "Developer", "Xcode", "macOS DeviceSupport"
    )
    for _, subdirs, _ in os.walk(macos_device_support):
        for subdir in subdirs:
            if os_version in subdir:
                return True
    return False


class TestRosetta(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessAppleSilicon
    @skipIfLLVMTargetMissing("X86")
    @skipIfDarwinEmbedded
    def test_rosetta(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")

        if rosetta_debugserver_installed():
            broadcaster = self.dbg.GetBroadcaster()
            listener = lldbutil.start_listening_from(
                broadcaster, lldb.SBDebugger.eBroadcastBitWarning
            )

            target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
                self, "Set a breakpoint here", self.main_source_file
            )

            event = lldb.SBEvent()
            os_version = get_os_version()
            if not has_rosetta_shared_cache(os_version):
                self.assertTrue(listener.GetNextEvent(event))
            else:
                self.assertFalse(listener.GetNextEvent(event))



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCppChainedCalls(TestBase):
    def test_with_run_command(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # Test chained calls
        self.expect_expr("get(set(true))", result_type="bool", result_value="true")
        self.expect_expr("get(set(false))", result_type="bool", result_value="false")
        self.expect_expr("get(t & f)", result_type="bool", result_value="false")
        self.expect_expr("get(f & t)", result_type="bool", result_value="false")
        self.expect_expr("get(t & t)", result_type="bool", result_value="true")
        self.expect_expr("get(f & f)", result_type="bool", result_value="false")
        self.expect_expr("get(t & f)", result_type="bool", result_value="false")
        self.expect_expr("get(f) && get(t)", result_type="bool", result_value="false")
        self.expect_expr("get(f) && get(f)", result_type="bool", result_value="false")
        self.expect_expr("get(t) && get(t)", result_type="bool", result_value="true")



"""
This is to make sure that the interrupt timer
doesn't influence synchronous user level stepping.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestStepVrsInterruptTimeout(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_step_vrs_interrupt(self):
        """This test is to make sure that the interrupt timeout
        doesn't cause use to flub events from a synchronous step."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        self.sample_test()

    def sample_test(self):
        """You might use the test implementation in several ways, say so here."""

        # This function starts a process, "a.out" by default, sets a source
        # breakpoint, runs to it, and returns the thread, process & target.
        # It optionally takes an SBLaunchOption argument if you want to pass
        # arguments or environment variables.
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )
        self.dbg.SetAsync(False)
        self.runCmd("settings set target.process.interrupt-timeout 1")
        thread.StepOver()
        self.assertState(
            process.GetState(), lldb.eStateStopped, "Stopped like we should"
        )



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects import linalg

from mlir.dialects.linalg.opdsl.lang import *

T1 = TV.T1
T2 = TV.T2


@linalg_structured_op
def pooling_poly(
    I=TensorDef(T1, S.N, S.H, S.W, S.C),
    K=TensorDef(T2, S.KH, S.KW, index_dims=[D.kh, D.kw]),
    O=TensorDef(U, S.N, S.OH, S.OW, S.C, output=True),
    reduce=BinaryFnAttrDef(default=BinaryFn.max_signed),
    cast=TypeFnAttrDef(default=TypeFn.cast_signed),
    strides=IndexAttrDef(S.SH, S.SW, default=[1, 1]),
    dilations=IndexAttrDef(S.DH, S.DW, default=[1, 1]),
):
    domain(D.n, D.oh, D.ow, D.kh, D.kw, D.c)
    O[D.n, D.oh, D.ow, D.c] = reduce[D.kh, D.kw](
        cast(U, I[D.n, D.oh * S.SH + D.kh * S.DH, D.ow * S.SW + D.kw * S.DW, D.c])
    )


with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

        # Pooling indexing maps.
        # CHECK: #[[$POOL_MAP_I:.+]] = affine_map<(d0, d1, d2, d3, d4, d5) -> (d0, d1 * 2 + d3, d2 * 4 + d4 * 2, d5)>
        # CHECK: #[[$POOL_MAP_K:.+]] = affine_map<(d0, d1, d2, d3, d4, d5) -> (d3, d4)>
        # CHECK: #[[$POOL_MAP_O:.+]] = affine_map<(d0, d1, d2, d3, d4, d5) -> (d0, d1, d2, d5)>

        # CHECK-LABEL: @test_f32i32_max_pooling
        # CHECK: linalg.generic
        # CHECK-SAME: indexing_maps = [#[[$POOL_MAP_I]], #[[$POOL_MAP_K]], #[[$POOL_MAP_O]]]
        # CHECK-SAME: iterator_types = ["parallel", "parallel", "parallel", "reduction", "reduction", "parallel"]
        # CHECK:      ^{{.*}}(%[[IN:.+]]: f32, %[[SHAPE:.+]]: f32, %[[OUT:.+]]: i32)
        # CHECK-NEXT:   %[[IN_CAST:.+]] = arith.fptosi %[[IN:.+]] : f32 to i32
        # CHECK-NEXT:   %[[MAX:.+]] = arith.maxsi %[[OUT]], %[[IN_CAST:.+]] : i32
        # CHECK-NEXT:   linalg.yield %[[MAX]] : i32
        # CHECK-NEXT: -> tensor<1x2x4x1xi32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((1, 4, 16, 1), f32),
            RankedTensorType.get((2, 2), f32),
            RankedTensorType.get((1, 2, 4, 1), i32),
        )
        def test_f32i32_max_pooling(input, shape, init_result):
            return pooling_poly(
                input, shape, outs=[init_result], strides=[2, 4], dilations=[1, 2]
            )

        # CHECK-LABEL: @test_f32i32_max_unsigned_pooling
        # CHECK:   = arith.fptoui
        # CHECK:   = arith.maxui
        @func.FuncOp.from_py_func(
            RankedTensorType.get((1, 4, 16, 1), f32),
            RankedTensorType.get((2, 2), f32),
            RankedTensorType.get((1, 2, 4, 1), i32),
        )
        def test_f32i32_max_unsigned_pooling(input, shape, init_result):
            return pooling_poly(
                input,
                shape,
                outs=[init_result],
                reduce=BinaryFn.max_unsigned,
                cast=TypeFn.cast_unsigned,
                strides=[2, 4],
                dilations=[1, 2],
            )

        # CHECK-LABEL: @test_f32f32_max_pooling
        # CHECK: linalg.generic
        # CHECK-SAME: indexing_maps = [#[[$POOL_MAP_I]], #[[$POOL_MAP_K]], #[[$POOL_MAP_O]]]
        # CHECK-SAME: iterator_types = ["parallel", "parallel", "parallel", "reduction", "reduction", "parallel"]
        # CHECK:      ^{{.*}}(%[[IN:.+]]: f32, %[[SHAPE:.+]]: f32, %[[OUT:.+]]: f32)
        # CHECK-NEXT:   %[[MAX:.+]] = arith.maximumf %[[OUT]], %[[IN:.+]] : f32
        # CHECK-NEXT:   linalg.yield %[[MAX]] : f32
        # CHECK-NEXT: -> tensor<1x2x4x1xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((1, 4, 16, 1), f32),
            RankedTensorType.get((2, 2), f32),
            RankedTensorType.get((1, 2, 4, 1), f32),
        )
        def test_f32f32_max_pooling(input, shape, init_result):
            return pooling_poly(
                input, shape, outs=[init_result], strides=[2, 4], dilations=[1, 2]
            )

        # CHECK-LABEL: @test_f32i32_min_pooling
        # CHECK:   = arith.fptosi
        # CHECK:   = arith.minsi
        @func.FuncOp.from_py_func(
            RankedTensorType.get((1, 4, 16, 1), f32),
            RankedTensorType.get((2, 2), f32),
            RankedTensorType.get((1, 2, 4, 1), i32),
        )
        def test_f32i32_min_pooling(input, shape, init_result):
            return pooling_poly(
                input,
                shape,
                outs=[init_result],
                reduce=BinaryFn.min_signed,
                strides=[2, 4],
                dilations=[1, 2],
            )

        # CHECK-LABEL: @test_f32i32_min_unsigned_pooling
        # CHECK:   = arith.fptoui
        # CHECK:   = arith.minui
        @func.FuncOp.from_py_func(
            RankedTensorType.get((1, 4, 16, 1), f32),
            RankedTensorType.get((2, 2), f32),
            RankedTensorType.get((1, 2, 4, 1), i32),
        )
        def test_f32i32_min_unsigned_pooling(input, shape, init_result):
            return pooling_poly(
                input,
                shape,
                outs=[init_result],
                reduce=BinaryFn.min_unsigned,
                cast=TypeFn.cast_unsigned,
                strides=[2, 4],
                dilations=[1, 2],
            )

        # CHECK-LABEL: @test_f32f32_min_pooling
        # CHECK:   = arith.minimumf
        @func.FuncOp.from_py_func(
            RankedTensorType.get((1, 4, 16, 1), f32),
            RankedTensorType.get((2, 2), f32),
            RankedTensorType.get((1, 2, 4, 1), f32),
        )
        def test_f32f32_min_pooling(input, shape, init_result):
            return pooling_poly(
                input,
                shape,
                outs=[init_result],
                reduce=BinaryFn.min_signed,
                strides=[2, 4],
                dilations=[1, 2],
            )


print(module)

with Context() as ctx, Location.unknown():
    module = Module.create()
    with InsertionPoint(module.body):
        f32 = F32Type.get()
        bool_t = IntegerType.get_signless(1)

        # CHECK: bool_max_unsigned_error: Unsupported 'max_unsigned' operands
        @func.FuncOp.from_py_func(
            RankedTensorType.get((1, 4, 16, 1), f32),
            RankedTensorType.get((2, 2), f32),
            RankedTensorType.get((1, 2, 4, 1), bool_t),
        )
        def test_bool_i1_max_unsigned_pooling_error(input, shape, init_result):
            try:
                pooling_poly(
                    input,
                    shape,
                    outs=[init_result],
                    reduce=BinaryFn.max_unsigned,
                    cast=TypeFn.cast_unsigned,
                    strides=[2, 4],
                    dilations=[1, 2],
                )
            except NotImplementedError as e:
                print(f"bool_max_unsigned_error: {e}")
            return init_result

        # CHECK: float_max_unsigned_error: Unsupported 'max_unsigned' operands
        @func.FuncOp.from_py_func(
            RankedTensorType.get((1, 4, 16, 1), f32),
            RankedTensorType.get((2, 2), f32),
            RankedTensorType.get((1, 2, 4, 1), f32),
        )
        def test_f32f32_max_unsigned_pooling_error(input, shape, init_result):
            try:
                pooling_poly(
                    input,
                    shape,
                    outs=[init_result],
                    reduce=BinaryFn.max_unsigned,
                    cast=TypeFn.cast_unsigned,
                    strides=[2, 4],
                    dilations=[1, 2],
                )
            except NotImplementedError as e:
                print(f"float_max_unsigned_error: {e}")
            return init_result
